#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# hint.pl
#
# retrieves a customer hint and displays it
#    written by GF 990521
#
#####
#
# Version 0.9.15 JC 20001003
#    secure sid
# Version 0.9.14 JC 20000913
#    oracle integration
# Version 0.9.12 TG 20000802
#    Removed check for email of 8 characters at a minimum for valid email
# Version 0.9.11 TG 20000801
#    Added colon to your hint
# Version 0.8.10 IB 20000717
#    om_render_language->om_render_lang
# Version 0.9.9 IB 20000607
#    u_email -> uname
# Version 0.9.8 IB 20000606
#    language and interface from table 'sessions'
# Version 0.9.7 IB 20000531
#    added custom-lib.pm
# Version 0.9.6 GF 20000523 
#    objclass changed to objtype 
# Version 0.9.5 GF 20000515 
#    added check for normal customer
# Version 0.9.4 GF 20000504
#    fixed error messages
# Version 0.9.3 GF 20000503
#    field names lowercase
# Version 0.9.2 GF 20000502
#    UID -> U_EMAIL
# Version 0.9.1 IB 20000501
#    introduced variable $script
# Version 0.7.28 JC 20000418
#    replaced loadtemplate with localize_template
#    included permute_include
#    removed $skeleton
# Version 0.7.27 GF 20000413
#    debug
# Version 0.7.26 GF 20000412
#    reformatted
#    changed UID to U_EMAIL, CUSTNUM to CUSTID
# Version 0.7.24 IB 20000411
#    default to language in site.pm
# Version 0.7.23 IB 20000410
#    multilingual messages
# Version 0.7.22 IB 20000402
#    new directory structure
# Version 0.7.21 IB 20000330
#    changed connect statement
# Version 0.7.20 JC 20000329
#    changed DBI prepare/connect
# Version 0.7.19 IB 20000327
#    corrected $relpath
# Version 0.7.18 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.17 JC 20000322
#    added colon to ssl_prefix
# Version 0.7.16 IB 20000317
#    renaming of templates
# Version 0.7.15 IB 20000302
#    updated copyright info
# Version 0.7.14 IB 20000226
#    changed store to site
# Version 0.7.13 JN 20000202
#    changed page redirection
# Version 0.7.12 JN 20000118
#    droped period from hint
# Version 0.7.11 JN 20000117
#    changed https string to store.pm's ssl_prefix
#    now using loadtemplate instead of old open()
#    got rid of !
# Version 0.7.10 JN 20000115
#    re-fixed ssl redirection
# Version 0.7.9 JN 20000107
#    fixed ssl redirect in nextcheckoutpg
# Version 0.7.8 RF 991203
#    added $site hashref pass via permutebit
# Version 0.7.7 IB 991203
#    added track
# Version 0.7.6 GF/IB 991202
#    added /site to /bits
# Version 0.7.5 IB 991126
#    changed bits/include to html/include
#    changed bits to bits/site
# Version 0.7.4 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.3 IB 991111
#    corrected the location of the DBI statement
# Version 0.7.2 IB 991109
#    removing of HACKS, and correcting of relpath
# Version 0.7.1 IB 991102
#    general cleanup
####

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in cgi-bin/om,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain cgi-bin
if ( $s !~ /cgi-bin/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "cgi-bin" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - cgi-bin is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/lib/cgi-lib.pm";
require "$relpath/lib/filetools.pm";
require "$relpath/conf/site.pm";
require "$relpath/lib/state.pm";
require "$relpath/lib/validate.pm";
require "$relpath/lib/localization.pm";
require "$relpath/lib/blowfisher.pm";

# must be the last require statement
require "$relpath/lib/custom-lib.pm";

## establish site-specific parameters
#
$site=setsite();

## parse the cgi elements
%FORM=();
&ReadParse;

## set now to time 
$now=time();

## set session id
#
&setsessionid();

## establish database connections
$q = "$site->{'db_site_config'}";
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});

$om_render_lang = "";
$interface = "";

$FORM{'sid'}=&os_unhash(pack(H16,$FORM{'sid'}), $site->{'hash_key'}) if ( $FORM{'sid'} );

if ($FORM{'sid'}) {
   $q="SELECT language,interface from sessions where sid=$FORM{'sid'} ";
   $sts = $dbh->prepare( $q );
   $sts->execute;
   $h=$sts->fetchrow_hashref;
   $om_render_lang=$h->{'language'};
   $interface=$h->{'interface'};
}

$om_render_lang=$site->{'om_locale_lang_def'} if ( ! $om_render_lang );
$om_render_lang="eng" if ( ! $om_render_lang );
$interface="html" if ( ! $interface );

## load messages
%script_msg=();          # user interaction messages
$script=( split /\./, $scriptname )[0];
&message;

## ssl default path
#
$sslpath="$site->{'sitesslurl'}/cgi-bin/om/ssl";

## determine identity of next page in checkout sequence
$q="SELECT sid,custid,mdate from sessions where sid=$FORM{'sid'} ";
$sts=$dbh->prepare( $q );
$sts->execute;
#$num=$sts->rows;
$sid_encrypt=unpack(H16, &os_hash( $FORM{'sid'}, $site->{'hash_key'} )); 
#if ( $num==1 ) {
#   ( $a,$b,$c ) = $sts->fetchrow;
if ( ( $a,$b,$c ) = $sts->fetchrow ) {
   if ( length($b)>2 ) {  # event - series
      # user is logged in
      $FORM{'nextcheckoutpg'}=$FORM{'pg_targ'};
  
   } else {
      # user not logged in
      $u="http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi";
      $u.="?sid=$sid_encrypt&pg_targ=$FORM{'pg_targ'}";
      $FORM{'nextcheckoutpg'}=$u;
   } # done checking on login when session registered

} else {  
   # this is an error
   $q="$site->{'ssl_prefix'}://$sslpath/checkout.cgi?sid=$sid_encrypt";
   $u="http://$site->{'siteurl'}/cgi-bin/om/welcome.cgi";
   $u.="?sid=$sid_encrypt&pg_targ=$q";
   $FORM{'nextcheckoutpg'}=$u;
} #/end of checking on sid for login

if ( $FORM{'email_inp'} ) {
   $FORM{'email_pre'}=$FORM{'email_inp'};  # keep the address populated
                                           # so the end user can see what they
                                           # entered
 
   # validate email address
   $u=validateemail ( $FORM{'email_inp'} );
   
   ## if this is not a correct email address 
   #  what is the correct error to display

   if ( $u eq "a" ) {
       # Error - No email address entered!
       $FORM{'email_err'}=$script_msg{3};
       
   } elsif ( $u eq "b" ) {
       # Error - Wrong email address format, need \@
       $FORM{'email_err'}=$script_msg{4};
       
   } elsif ( $u eq "c" ) {
       # Error - Invalid email address!
       $FORM{'email_err'}=$script_msg{2};
       
       ## the email address is in the correct format, see if we have a 
       #  record on file for the user
       
   } else { 
       $q="SELECT uname,hint from contacts where uname='$FORM{'email_inp'}' ";
       $q.=" and objtype='39003-A'";
       $sts = $dbh->prepare ( $q );
       $sts->execute;
       
#       $num=$sts->rows; # $num should be 1 to continue
#       ($uname,$thehint)=$sts->fetchrow;     

       if ( ($uname,$thehint)=$sts->fetchrow ) {
          $num=1;
       }
       
       $sts->finish if ($sts);
       $dbh->disconnect if ( $dbh ); 
       
       if ( $num==1 ) {
	   ## is there a hint on file for this account
	   #
            if ( length($thehint)>1 )  {
		$FORM{'hint_err'}=$script_msg{1}.$thehint; 
		
            } else {
		$FORM{'hint_err'}="No hint on file for this account!";
            } #/done looking for hint on file
	    
	} else {
	    $FORM{'email_err'}=$script_msg{2};
	} #/done checking if we have a record or not
   }
   
} #/done with looking up hint 

## start the response header
#
print "Content-type: text/html\n\n";

## take elements off page and store in $thispage
#
$thispage = localize_template("page_hint_lookup",$om_render_lang,$interface,0);

## fork and track the user 
#
`$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/cgi-bin/om/track.cgi`;

## create the page stdout
#
$thispage = permutebit ( $thispage, \%FORM );
$thispage = permutebit ( $thispage, $site );
$thispage = permuteclean ( $thispage );
$thispage = permute_include( $thispage, $om_render_lang, "html", 0);

print $thispage;

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

   if ($om_render_lang eq "eng" ) {
      $script_msg{1}="Your hint is: ";
      $script_msg{2}="Error - Invalid email address!";
      $script_msg{3}="Error - No email address entered!";
      $script_msg{4}="Error - Wrong email address format, need \@";
      $script_msg{5}="No hint on file for this account!";
      
   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
 
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }

     close(EM);
  }
}

## provide a return value
#
1;
