#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# generate_temps.pl
#
# Program to generate temporary data files
#    to feed into various general pulldowns
# 
# Version 0.9.15 TG 20000804
#    called exdbi for pulldowns
# Version 0.9.14 JC 20000717
#    took out not specified
# Version 0.9.13 JC 20000628
#    added warehouse
# Version 0.9.12 JC 20000621
#    final corrections thanks to Tara
# Version 0.9.11 JC 20000620
#    corrections
# Version 0.9.10 JC 20000620
#    added non brand support
# Version 0.9.9 JC 20000616
#    changed ln command and
#    linking 47004-A to SKU_SERIES
# Version 0.9.8 JC 20000615
#    by default will put in "Not Specified"
# Version 0.9.7 JC 20000614
#    removed debug
# Version 0.9.6 JC 20000614
#    put in early exit for non argument case
# Version 0.9.5 JC 20000613
#    major rewrite to use objtype instead of
#    brand / others
# Version 0.9.4 IB 20000531
#    added custom-lib.pm
# Version 0.9.3 RF 20000523
#    refitted for new data structure and classes
#    note - this is not generalized properly, but to work with
#       the expectations of the various administrative scripts
#       prior to their pending deprecation in favor of admin_obj 
# Version 0.9.2 IB 20000502
#    corrected the sequence of statements
# Version 0.9.1 IB 20000426
#    introduced $script variable
# Version 0.7.15 JC 20000412
#    removed AUTHOR
# Version 0.7.14 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.13 JC 20000411
#    default to eng if no default lang
# Version 0.7.12 JC 20000410
#    multilingual support
# Version 0.7.11 IB 20000402
#    new directory structure
# Version 0.7.10 JC 2000330
#    changed connect statement
# Version 0.7.9 JC 20000328
#    changed DBI prepare/connect
# Version 0.7.8 IB 20000327
#    corrected $relpath
# Version 0.7.7 IB 20000323
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.6 IB 20000301
#    updated copyright
# Version 0.7.5 IB 20000229
#    changed table to database
# Version 0.7.4 IB 20000223
#    changed the name store.pm to site.pm
# Version 0.7.3 IB 991217
#    eliminated ERLOG
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    calculation of a relative path
#    using of new global variables
# Version 19 RF 990908
#    relative path symlinks - removed absolute symlink paths
#    changed libraries to relative paths
#    added test for handler-less sku
#
# Version 12 SH 990707
#    moved to system 6 data structure and queries
#    if encoding does not have relevant field, sku.sel is created
# Version 11 GF 990310
#    fixed problem regarding nobody running the script
# Version 10 RF 981219
#    removed refs to $datahost
#    removed all calls to machine.pm
# Version 9 RF 981127
#    fixed generation of pulldowns
# Version 8 RF 981124
#    added fieldname binding detection, generates fieldname.sel
#    files as appropriate in addition to XEC sku filenames
# Version 7 RF 981123
#    takes a storecode as an argument
# Version 6 RF 981122
#    multi-store capacity and bits/sel useage
# Version 5 RF 981111
#    converted to DBI interface
# Version 4 RF 981103 
#    added sku.sel generation for encodings
# Version 3 RF 981103 lost
# Version 2 RF 981013
#    added string conversion from mysql to text
# Version 1 RF 981012
#

## determine path from store root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store (site) root

$relpath .= "/../$siteshortname";

use DBI;

require "$relpath/lib/formtools.pm";
require "$relpath/conf/site.pm";
require "$relpath/lib/custom-lib.pm";

## establish site-specific parameters
#
$site=setsite();
$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

$verbose_errors=1;
%script_err=();
$script=( split /\./, $scriptname )[0];
&message;

## establish connection in
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}); 

$datapath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$om_render_lang/html/sel";

chdir( $datapath );          # move into the datapath directory

if ( ! $ARGV[0] ) { 
   print STDERR "$scriptname: $script_err{2} \n";
   exit;
} 

$objid = $ARGV[0];

## generate pulldown data file
#
open(FH, ">$datapath/$objid.sel") || die "$script_err{1} $datapath/$objid.sel";

## sku_series (brands) pulldown ( in things )
# 

if ( $objid eq "47004-A" ) {

   $q="SELECT objid,title from things WHERE objtype='$objid' ";
   $q.=" AND status='15000-A' ";
   $q.=" ORDER BY title ";
   $sth=$dbh->prepare($q);
   $sth->execute;
#   $num=$sth->rows;
   print FH "| Not Specified\n";
#      for ($x=0; $x<$num; $x++ ) {
#         ($k,$v) = $sth->fetchrow_array;
#         $v=string_exmysql($v);
   while ( ($k,$v) = $sth->fetchrow_array ) {
         $v=string_exdbi($v);
         print FH "$k\| $v \n";
      }

   $q="SELECT title from things WHERE objid='$objid'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   $title = $sth->fetchrow;


## other pulldowns (ones in extraprop)
#


} 
elsif ( $objid eq "74003-A" ) {

   $q="SELECT objid,title from things WHERE objtype='$objid' ";
   $q.=" AND status='15000-A' ";
   $q.=" ORDER BY title ";
   $sth=$dbh->prepare($q);
   $sth->execute;
#   $num=$sth->rows;
#   print FH "| Not Specified\n";
#      for ($x=0; $x<$num; $x++ ) {
#         ($k,$v) = $sth->fetchrow_array;
#         $v=string_exmysql($v);
   while ( ($k,$v) = $sth->fetchrow_array ) {
         $v=string_exdbi($v);
         print FH "$k\| $v \n";
      }

   $q="SELECT title from things WHERE objid='$objid'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   $title = $sth->fetchrow;


## other pulldowns (ones in extraprop)
#
}

else {

   $q="SELECT body from extraprop WHERE objid='$objid' AND fieldname='sel_name'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   $f = $sth->fetchrow;

   $q="SELECT body from extraprop WHERE objid='$objid' AND fieldname='encoding'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   print FH "| Not Specified\n";
   $v = $sth->fetchrow;
   $v =~ s/\r//g;
   $v = string_exdbi($v);
   @pulldown=split(/\n/,$v); 
   for ($x=0; $x<=$#pulldown; $x++) {
   print FH "$pulldown[$x] \n";
   }
   $q="SELECT title from things WHERE objid='$objid'";
   $sth=$dbh->prepare($q);
   $sth->execute;
   $title = $sth->fetchrow;  
   $title=$f if $f;
}

close(FH);

$title =~ tr/A-Z/a-z/;
$title =~ s/\s//g;
$title =~ s/\\//g;

if ( $objid eq "47004-A" ) {

   `/bin/ln -fs 47004-A.sel sel_sku_series.sel`;
}
elsif ($objid eq "74003-A") {
   `/bin/ln -fs 74003-A.sel sel_warehouse.sel`;

} elsif ( $title =~ /^(sel\_)/ ) {

   `/bin/ln -fs "$objid.sel" "$title.sel"`;  

} else {

   `/bin/ln -fs "$objid.sel" "$title.sel"`;

}

$sth->finish;
$dbh->disconnect if ( $dbh );

##### EnGarde runs Apache as user webd
#
`chgrp webd $datapath/*.sel`;
`chmod 775 $datapath/*.sel`;

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below
   if ( $verbose_errors ) {
      if ($om_render_lang eq "eng" ) {
         $script_err{1}="Unable to open";
         $script_err{2}="Argument needed";
      } else {
         open( EM, "<$relpath/bits/$om_render_lang/system/$script.errors");
         while(<EM>) {
            chomp;
            next if /^\#/;
            next if (! $_);
            @temp=split /\t/;
            $script_msg{$temp[0]}=$temp[1];
         }
         close(EM);
      }
   }
} 

1;
