#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# run_stats.pl
# 
# Create daily summary report on core statistics
# Version 1 NC 20000929
#    space between titles of variants
# Version 0.7.40 NC 20000919
#    mark voided orders using orderid instead of sid
# Version 0.7.39 NC 20000911
#    ->rows
# Version 0.7.38 JC 20000910
#    ->rows
# Version 0.7.37 NC 20000829
#    get rid of voided orders
# Veriosn 0.7.36 NC 20000825
#    fixed reports and add the number and percentage for visitors left 
# Version 0.7.35 JC 20000823
#    ->rows
# Version 0.7.34 NC 20000818
#    fixed reports by adding back the missing contents and editing some
#    field.
# Version 0.7.32 IB 20000729
#    fixed reports
# Version 0.7.32 IB 20000727
#    correction to order totatls (exclude incomplete orders)
# Version 0.7.31 IB 20000727
#    correction to shipping cost
# Version 0.7.30 IB 20000726
#    shipping costs
# Version 0.7.29 IB 20000724
#    changed the negative value for no ordering
# Version 0.7.28 IB 20000719
#    FORM variables were not being filled before the second pass
# Version 0.7.27 IB 20000717
#    correct the location of the report for cron jobs
# Version 0.7.26 IB 20000711
#    TOP20->TOP25
# Version 0.7.25 IB 20000710
#    objid -> lineobjid
# Version 0.7.24 IB 20000707
#    msku -> objid
# Version 0.7.23 IB 20000706
#    removed EE1 
#    corrected blank line in the report
# Version 0.7.22 IB 20000702
#    corrected report
# Version 0.7.21 IB 20000606
#    deleted redundant comment lines
# Version 0.7.20 IB 20000603
#    further conversion
# Version 0.7.19 IB 20000602
#    converted to Rel. 1.0.1
# Version 0.7.18 IB 20000531
#    added custom-lib.pm
# Version 0.7.17 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.16 JC 20000411
#    default to eng if no default lang
# Version 0.7.15 JC 20000330
#    changed connect statement
# Version 0.7.14 JC 20000328
#    changed DBI prepare/connect
# Version 0.7.13 IB 20000327
#    corrected $relpath
# Version 0.7.12 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.11 IB 20000316
#    new directory structure
# Version 0.7.10 IB 20000302
#    removed -w
# Version 0.7.9 IB 20000301
#    updated copyright
# Version 0.7.8 IB 20000229
#    changed table to database
# Version 0.7.7 IB 20000223
#    changed the name store.pm to site.pm
# Version 0.7.6 JZ 20000215
#    display actual checked out orders, use templates
# Version 0.7.5 JZ 20000207
#    corrected the numbers
# Version 0.7.4 IB 20000204
#    changed "Snapshot view" to "Snapshot View", deleted obsolete summary
# Version 0.7.3 JZ 20000126
#    print to file
# Version 0.7.2 IB 991112
#    corrected the determining of a relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    removing of HACKS, and correcting of relpath
# Version 2 RF 991108
#    rearangement of hash ordering and prelim dump
# Version 1 RF 991104
#    creation
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/conf/site.pm";
require "$relpath/lib/filetools.pm";
require "$relpath/lib/localization.pm";

# must be the last require statement
require "$relpath/lib/custom-lib.pm";

## establish site-specific parameters
#
$site=setsite();

$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

## establish connection in
#
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'}); 

$targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/data/stats";
$skeletonpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/bits/$site->{'om_locale_lang_def'}/admin/";
## Load templates for output HTML
#
#%template=();

########### general flow
#
# validation of input args ( can be order list )
#    fill in defaults for store/date
#
# select all sessions for date
#    determine sessions per referer
#    determine sessions per adbuy
#    determine sessions per artwork
#    determine sessions per affiliate
#    determine sessions per browser
# 
# select all pageviews
#    determine pagedepth per session
#    determine total number of page views
#
# select all orders
#    determine total revenues
#    determine total line items
#    determine total items
#    determine total shipping
#    determine total tax
#
# Derivative Values
#    average pagedepth = pageviews/sessions
#    conversion rate = sessions/orders
#    

## initialize tallying hash
#
%tot=();
$TOP25 = 25;
$days = 0;
$NVP{'snap_visleft'} = 0;
$NVP{'snap_checkout'} = 0;
$NVP{'snap_left'} = 0;
$NVP{'snap_leftorder'} = 0;
$NVP{'topitems'} = "";
$NVP{'snap_newjoin'}=0;
$NVP{'snap_noorder'}=0;
## construct timestamps
#
if ( $#ARGV >= 2 ) {
   # there was a second argument, see if it is numeric...
   # TBD - what formats are acceptable?
   #	$test_str = $ARGV[1];
#	$year  = $test_str =~ m/.{4}$/  ;
	
#	if (($year < 1970) or ($year > 2100)) {
#			print "ERROR: invalid year format yyyymmdd\n";
#			exit 0;
#		}
### for now the number is the int days back
	$days = $ARGV[2];
}

# figure out the user who generated this file, default is cron
if ($ARGV[0]) {
	$statuser = $ARGV[0];
} else {
	$statuser = 'cron';
}

# by default, this will run by cron and it does not have any argument
if ($ARGV[1]) {
	$time_n=$ARGV[1];
} else {
	$time_n=time();
}

# we do the stats every 24 hours in default 3:00am in the morning
## construct timestamp of midnight last
#
for (0..$days){

	if ($time_i) {$time_n = $time_i - (24 * 60 * 60);}
#	@ltime=localtime($time_n);
	$time_f=$time_n ; 
#	if ( ($ltime[0]+$ltime[1]+$ltime[2])!= 0 ) {
#		$time_i=$time_n - $ltime[0] - ( $ltime[1]*60 ) - ( $ltime[2]*3600 )  ;
#	} # done testing for midnight
## construct timestamp of midnight next
        $time_i=$time_f - (24 * 60 * 60);       # midnights are always 24hours apart


#search for employees (NC)
       $q="SELECT objid, objtype, cdate FROM contacts";
       $sth= $dbh->prepare ( $q );
       $sth->execute;
#       $num=$sth->rows;
#       foreach (1..$num) {
       while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {
            if ($h->{'objtype'} eq '39004-A'){
               $employee{$h->{'objid'}}=1;  # Mark the employees

            } elsif ( $h->{'cdate'} >= $time_i && $h->{'cdate'} < $time_f ) {
               $NVP{'snap_newjoin'}++; # count new customers
            }
     
       }

        $q="SELECT sid FROM clicks where cdate>=$time_i and cdate<$time_f";
        $sth= $dbh->prepare ( $q );
        $sth->execute;
#        $num=$sth->rows;
#        foreach (1..$num) {
        while ( my $sid = $sth->fetchrow_array ) {
            $clicks{$sid}=1;
        }

## sessions
#
# initialize session hash
	%session=();
# initialize counter hashes
	%ses_ref=();               # by referer
	%ses_adb=();               # by adbuy
	%ses_adb_pur=();           # made purchase
	%ses_art=();               # by banner artwork
	%ses_art_pur=();           # made purchase
#	%ses_aff=();               # by affiliate code
	%ses_bro=();               # by browser

# 	%ses_newu=();		       # by sid w/o UserID        (NC)            
# 	%ses_oldu=();		       # by sid w UserID          (NC)            
# 	$ses_newu{'TOTAL'} =0;                                    (NC)            
# 	$ses_oldu{'TOTAL'} =0;                                    (NC)            
                                           
        $ses{'log'}=0;               # total logged in visitors (NC)
        $ses{'nolog'}=0;             # total not logged in visitors (NC)
        $totvisitors = 0;            # total visitors (NC)
        $tot_bro=0;                  # total browsers (NC)
 
        %visitors=();                # hash holds visitors' sid

	$q="SELECT \* from sessions where cdate>=$time_i and cdate<$time_f ";
	$sth = $dbh->prepare( $q );
	$sth->execute;
#	$num=$sth->rows;
#	$tot{'sessions_raw'}=$num;       # total sids burned through
	$tot{'sessions_ok'}=0;           # ignoring sessions from within site
#	foreach ( 1..$num ) {
	while (	$h = $sth->fetchrow_hashref(NAME_lc) ) {

                # Get rid of employees' stuff and mark sid which is run by employees
                if ($employee{$h->{'custid'}}) {           
                    $employee{$h->{'sid'}}=1; # employees' sid
                    next;
                }
	
                	
		## filter out internally referred sessions
		#
		#next if ( $h->{'referer'} =~ /($site->{'siteurl'}|$site->{'sitesslurl'})/ );
		$tot{'sessions_ok'}++;
		
		## store session minimum data
		#
		$session{$h->{'sid'}}=$h;         # HACK - this will explode for large sessions...
		
		## test for existence of counters
		#

# (NC)          if ( $h->{'sid'} ) {   
                if ( $h->{'sid'} && $clicks{$h->{'sid'}} ){ # count customers with at least one click (NC) 

# (NC)                  if ( ! $ses_newu{$h->{'sid'}} ){
# (NC)                     $ses_newu{$h->{'sid'}} = 1;
# (NC)                     $ses_newu{'TOTAL'}++;
# (NC)                  }	

                    $visitors{$h->{'sid'}}=1; # mark the visitors sid            
     
                    #Count toal visitors, login, and not login visitors (NC)     
                    $totvisitors ++;
        
                    if ($h->{'custid'}){
                       $ses{'log'} ++;
                    }
                    else {
                       $ses{'nolog'} ++;
                    }  

		

		$ses_ref{$h->{'referer'}}++ if ( $h->{'referer'} );     # by referer
		$ses_adb{$h->{'adbuy'}}++ if ( $h->{'adbuy'} );         # by adbuy code
		$ses_art{$h->{'artwork'}}++ if ( $h->{'artwork'} ) ;    # by banner ad artwork
		$ses_bro{$h->{'browser'}}++ if ( $h->{'browser'} ) ;    # by customer browser

                $tot_bro++ if ($h->{'browser'});   # Count total browsers (NC)
               }
                ## TBD - add affiliate counting
	} # done with sessions
## start in on order lines (search ocart)

# login orders are placed in orders
# checked out orders are placed in ocart
# orders that are not checked out are placed in tcart

        # search for voided orders
        my %voided=();

        $q="SELECT objid from orders where cdate>=$time_i and cdate<$time_f and (status = '15028-A' or status = '15029-A')";
        $sth = $dbh->prepare( $q );
        $sth->execute;
#        $num=$sth->rows;

#        foreach (1..$num) {
        while ( my $objid = $sth->fetchrow_array ) {
           $voided{$objid}=1;
        }

	%tot_sku=();
	%sku_price=();
        %tot_objid=();
        %objid_price=();
        %tot_lineobjid=();
        %lineobjid_price=();
        %inocart=();             # hash holds the order sid in ocart
 
	$q="SELECT \* from ocart where cdate>=$time_i and cdate<$time_f and objclass != '140-A'";
	$sth = $dbh->prepare( $q );
	$sth->execute;
#	$num=$sth->rows;
        $tot{'orders_shp'}=0;          # total shipping paid
#	$tot{'orders_raw'}=$num;       # total pageviews burned through
#	foreach ( 1..$num ) {
        while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {

           # Get rid of employees' stuff
           next if $employee{$h->{'orderid'}};
     
           $inocart{$h->{'sid'}}=1;

           # get rid of voided orders
           next if $voided{$h->{'orderid'}};

           $checkedout{$h->{'orderid'}} =1;
           $tot_lineobjid{$h->{'lineobjid'}} = 0 if ( ! $tot_lineobjid{$h->{'lineobjid'}});
           $tot_lineobjid{$h->{'lineobjid'}} += $h->{'quant'} ;

           $lineobjid_price{$h->{'lineobjid'}} = $h->{'unitpr'} if ( ! $lineobjid_price{$h->{'lineobjid'}});

		# next if (! $orders{$h->{'sid'}});  #order not checked out
		$tot_sku{$h->{'msku'}} = 0 if ( ! $tot_sku{$h->{'msku'}});
                $tot_sku{$h->{'msku'}} += $h->{'quant'} ;
                $tot_objid{$h->{'objid'}} = 0 if ( ! $tot_objid{$h->{'objid'}});
                $tot_objid{$h->{'objid'}} += $h->{'quant'} ;

                $sku_price{$h->{'msku'}} = $h->{'unitpr'} if ( ! $sku_price{$h->{'msku'}});
                $objid_price{$h->{'objid'}} = $h->{'unitpr'} if ( ! $objid_price{$h->{'objid'}});

		next if ( ! $h->{'sid'} );
		if ($session{$h->{'sid'}}->{'adbuy'}) {
			if (! $ses_adb_pur{ $session{$h->{'sid'}}->{'adbuy'} }) {
		 		$ses_adb_pur{ $session{ $h->{'sid'} }->{'adbuy'} } =0;
			} 
			$ses_adb_pur{$session{$h->{'sid'}}->{'adbuy'}}++; 
			if (!$tot_adb{$session{$h->{'sid'}}->{'adbuy'}}) {
				$tot_adb{$session{$h->{'sid'}}->{'adbuy'}} = 0 ;
			}
			$tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'unitpr'}
		}	
		if ($session{$h->{'sid'}}->{'artwork'}) {
			if (! $ses_art_pur{ $session{$h->{'sid'}}->{'artwork'} }) {
		 		$ses_art_pur{ $session{ $h->{'sid'} }->{'artwork'} } =0;
			} 
			$ses_art_pur{$session{$h->{'sid'}}->{'artwork'}}++; 
		}	
	} # done with order line items

        $q="SELECT \* from ocart where cdate>=$time_i and cdate<$time_f and objclass = '140-A'";
        $sth = $dbh->prepare( $q );
        $sth->execute;
#        $num=$sth->rows;
        $tot{'orders_shp'}=0;          # total shipping paid
#        foreach ( 1..$num ) {
        while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {

           # Get rid of employees' stuff
           next if $employee{$h->{'sid'}};

           # get rid of voided orders
           next if $voided{$h->{'orderid'}};

           $tot{'orders_shp'} += $h->{'linetotal'};
        }

#start in tcart
#number of people left stuff in cart and left site, search tcart
        %intcart=();       # hash holds orders sid in tcart 

        $q = "SELECT sid from tcart where objclass!='140-A' and cdate>=$time_i and cdate<$time_f";    #Just count orders without shippings (NC)

        $sth = $dbh->prepare( $q );
        $sth->execute;
#        $num=$sth->rows;
#        foreach (1..$num) {
        while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {

           # Get rid of employees' stuff
           next if $employee{$h->{'sid'}};

           $intcart{$h->{'sid'}} =1;  # mark the order sid in tcart
        }

        #foreach (keys %intcart) { $NVP{'snap_left'}++;}


#start on orders
#
        %tot_adb=();
        %inorders=();   # hash holds orders sid in orders

        $q="SELECT \* from orders where cdate>=$time_i and cdate<$time_f";
        
        $sth = $dbh->prepare( $q );
        $sth->execute;
#        $num=$sth->rows;
        $tot{'orders_num'}=0;
        $tot{'orders_act'}=0;          # actual count of real orders
        $tot{'orders_tax'}=0;          # total sales tax collected
        $tot{'orders_mds'}=0;          # total merchandise sold
        $tot{'orders_tot'}=0;          # total charged for orders


#       $t = loadtemplate("row_stats_sid", $skeletonpath);
#       $t =~ s/<!--.*-->\n>//s;

#        foreach ( 1..$num ) {
        while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {
              
                # Get rid of employees' stuff
                next if $employee{$h->{'sid'}};

                $inorders{$h->{'sid'}}=1;     # mark the order sid in orders
          
                # get rid of voided orders
                next if $voided{$h->{'objid'}};

           if ($h->{'status'} ne '15012-A') {
                $tot{'orders_num'}++;  # total orders with checkout and not checkout


               # if (! $checkedout{$h->{'objid'}}) { #not checked out orders, stop right before send button
               #        $NVP{'snap_leftorder'} ++;
               #        next;
               #}

                #$orders{$h->{'sid'}} = 1;
                #$NVP{'snap_checkout'}++;

                ## TBD - exclude uncompleted orders
 
                ## count revenue for the session
                #
                $session{$h->{'sid'}}->{'revenue'}=$h->{'ordtotal'};

# stats according to order by sid
#               $SIDH{'SID'} = $h->{'SID'};
#               $SIDH{'SIDREF'} = "http://$site->{'storeserver'}$site->{'storedomain'}/stats/$h->{'SID'}\.html";
#               $SIDH{'ORDERDATE'} = scalar localtime($h->{'CDATE'});
#               $SIDH{'OID'} = $h->{'OID'};
#               $SIDH{'OIDREF'} = "/cgi-bin/om/admin/order.cgi?&sid=$h->{'SID'}&mode=showorder&loadfrom=db&orderid=$h->{'OID'}";
#               $SIDH{'SHIPTO'} = $h->{'SHIPTO'};
#               $SIDH{'ORDERTOTAL'} = $h->{'ORDTOTAL'};
#               $NVP{'ORDERSID'} .= permutebit($t, \%SIDH);
 
                if ($session{$h->{'sid'}}->{'adbuy'}) {
                        $tot_adb{$session{$h->{'sid'}}->{'adbuy'}} += $h->{'ordtotal'};
                }
                if ($session{$h->{'sid'}}->{'artwork'}) {
                        $tot_art{$session{$h->{'sid'}}->{'artwork'}} += $h->{'ordtotal'};
                }
                $tot{'orders_tax'}+= $h->{'st_tax'};
                $tot{'orders_mds'}+= $h->{'st_taxable'} + $h->{'st_nontax'};

# (NC)          $tot{'orders_tot'}+= $h->{'ordtotal'};
                $tot{'orders_tot'}+= $h->{'st_taxable'};  #Order total include product total and tax total (NC)
            } 
        } # done with orders     

## Start to count visitors left status -- with empty cart, with stuff in cart, just before completing order, and purchased.
    foreach (keys %visitors){
        if ($inocart{$_}) {
             $NVP{'snap_checkout'}++;
        } else {
             $NVP{'snap_visleft'}++; 
        }

        if (!$intcart{$_} && !$inocart{$_}){
             $NVP{'snap_noorder'}++;
        }

       if ($intcart{$_} && !$inorders{$_}) {
             $NVP{'snap_left'}++;
       } elsif ($intcart{$_} && $inorders{$_}) {
             $NVP{'snap_leftorder'}++;
       }          
    }
           


## start in on clicks
#
        %tot_ref=();
        %tot_req=();
        %geom=();


# (NC)  $q="SELECT \* from clicks where cdate>=$time_i and cdate<$time_f";

        $q="SELECT \* from clicks where cdate>=$time_i and cdate<$time_f and request like '%.shtml'";  # Count pages at store side (NC)

        $sth = $dbh->prepare( $q );
        $sth->execute;
#        $num=$sth->rows;
        #$tot{'pageviews_raw'}=$num;       # total pageviews burned through
        $tot{'pageviews_raw'}=0;
        $tot{'pageviews_sid'}=0;          # pageviews with session id known
        $tot{'homepage'}=0;
        $tot{'refer'}=0;
        $tot{'request'}=0;

#        foreach ( 1..$num ) {
        while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {

                # Get rid of employees' stuff
                next if $employee{$h->{'sid'}};

                $tot{'pageviews_raw'}++;  # total pageviews burned through

                $tot{'homepage'}++ if ( $h->{'request'} =~ /html\/$om_render_lang\/home.shtml/ || $h->{'request'} eq "\/");
                @tmp = split(/\?/, $h->{'request'});
                $tot_req{$tmp[0]} ++;
                $tot{'request'}++;

                if( $h->{'referer'} !~ /$site->{'siteurl'}/ && $h->{'referer'} !~ /$site->{'sitesslurl'}/ ) {
                        @tmp = split(/\?/, $h->{'referer'});
                        $tot_ref{$tmp[0]} ++ if $h->{'referer'};
                        $tot{'refer'}++;


                }
                next if ( ! $h->{'sid'} );

                #figure out the geometery
                if ($orders{$h->{'sid'}}) {  #this click has at least an order, retrieve product
                        $geom{$h->{'sid'}}->{$h->{'cdate'}}->{'referer'}= $h->{'referer'};
                        $geom{$h->{'sid'}}->{$h->{'cdate'}}->{'request'}= $h->{'request'};
                }

                $tot{'pageviews_sid'}++;
                $session{$h->{'sid'}}->{'clicks'}=0 if (!$session{$h->{'sid'}}->{'clicks'});

                $session{$h->{'sid'}}->{'clicks'}++;
        } # done with pageviews

#Get title for each variant from extraprop (NC)
        %vtitle=();
        $q="SELECT objid, body from extraprop WHERE fieldname='encoding' and body like '%:%'";
        $sth = $dbh->prepare($q);
        $sth->execute;
#        $num=$sth->rows;
#        foreach (1..$num) {
        while ( ($objid, $body) = $sth->fetchrow_array ) {
               my @titles = split (/\n/, $body);
               foreach (@titles) {
                   (my $k, $v) = split (/:/, $_);
                   $vtitle{$objid}->{$k}=$v;
               }
        }

#search for titles
	%title=();
        %axisnames=();
        $q="SELECT objid, msku, title, axisnames, vcode from things";
	$sth = $dbh->prepare( $q );
	$sth->execute;
#        $num=$sth->rows;
#	foreach (1..$num) {
	while (	$h = $sth->fetchrow_hashref(NAME_lc) ) {
                
                 # Load titles for each page
                 
                 $title{$h->{'msku'}} = $h->{'title'} if ($h->{'title'});
                 if ($h->{'title'}) {
                      $title{$h->{'objid'}} = $h->{'title'};}
                 else {
                      $title{$h->{'objid'}} = $title{$h->{'msku'}};}

                 if ($h->{'axisnames'}){
                    $axisnames{$h->{'msku'}}=$h->{'axisnames'};  
                 }
                 if ($h->{'vcode'}) {
                         my @axis = split (/:/, $axisnames{$h->{'msku'}});
                         my $n = (my @vcode = split (/:/, $h->{'vcode'}));
                         $title{$h->{'objid'}} .= " --"; 
                         for ($i=0; $i<$n; $i++){
                             $title{$h->{'objid'}} .= " $vtitle{$axis[$i]}->{$vcode[$i]}";
                         }                 
                 }
      }	

# Customers newly joined
#  (NC)  $q="select count(*) from levents where dvalue like '\%\tCreate CustNum\%' and cdate>=$time_i and cdate<$time_f ";
#         $q="select count(*) from contacts where cdate>=$time_i and cdate<$time_f ";
#         $sth = $dbh->prepare( $q );
#         $sth->execute;
#         ($NVP{'snap_newjoin'}) = $sth->fetchrow;

## Output something...
#
        $fromtime = scalar localtime($time_i);
        $totime = scalar localtime($time_f);
        @fd = localtime($time_f);
        $fd[4] += 1;
        if ($fd[4] <10) { $filemonth = "0".$fd[4]; }
        else { $filemonth = $fd[4] ;}

        $fd[3] = "0".$fd[3] if $fd[3] <10;
        $filedate = (1900 + $fd[5]).$filemonth.$fd[3];

        # check if the file for the day already exist
        opendir (DIR, $targetpath);
        @dirfiles = readdir (DIR);
        close (DIR);

        #check if cron and include directory exist
	foreach $d (@dirfiles) {
		if ($d eq 'cron') {
			$cronexist = 1;
		} elsif ($d eq 'include') {
			$includeexist = 1;
		}
	}

	if (! $cronexist) { system("mkdir $targetpath/cron"); }
	if (! $includeexist) { system("mkdir $targetpath/include"); }

#now create the sid geometery file
#        foreach $sid (sort keys %geom) {
#                $geomstring="<html><body bgcolor=#FFFFFF> <p><table width=100% border=1>\n";
#                $geomstring .= "<tr><td width=20%>DATE</td><td width=55%>PAGE</td><td width=25%>GEOMETRY</td></tr>\n";
#                foreach $cdate (sort keys %{$geom{$sid}}) {
#                        $geomstring .= "<tr><td width=20%>";
#                        $geomstring .= scalar localtime($cdate);
#                        $geomstring .= "</td><td width=55%>  $geom{$sid}->{$cdate}->{'REQUEST'} </td>";
#                        $gif = getgifname($geom{$sid}->{$cdate}->{'REFERER'});
#                        $geomstring .= "<td width=25%><img src=http://$site->{'storeserver'}$site->{'storedomain'}/stats/images/$gif\.gif </td></tr>\n";
#                }

#                $geomstring .= "</table></body></html>";
#                open (G, ">$targetpath/$sid\.html");
#                print G $geomstring;
#                close (G);
#        }

        $snapflag = 1;
        if ($statuser eq "cron") { # cron time
                $snapflag = 0;
                $targetpath .= "/cron";
        } else {
                $targetpath .= "/include";
        }

	#snapshot
# (NC) 	$totvisitors = $ses_newu{'TOTAL'} + $ses_oldu{'TOTAL'};       

	$NVP{'fromtime'} = $fromtime;
	$NVP{'totime'} =  $totime;
	$NVP{'statsuser'} = $statuser;
        

        $NVP{'snap_page'} = $tot{'pageviews_raw'};
        $NVP{'snap_homepage'} = $tot{'homepage'};
        $NVP{'homepagep'} = 0;
        $NVP{'homepagep'} = cutnumber($tot{'homepage'} / $tot{'pageviews_raw'} *100) if $tot{'pageviews_raw'};
	$NVP{'snap_visitor'} = $totvisitors; 

# (NC)	$NVP{'snap_log'} = $ses_oldu{'TOTAL'};
# (NC)  $NVP{'logp'} = 0;
# (NC)  $NVP{'logp'} = cutnumber($ses_oldu{'TOTAL'} / $totvisitors *100) if $totvisitors;
# (NC)	$NVP{'snap_nolog'} = $ses_newu{'TOTAL'};
# (NC)  $NVP{'nologp'} = 0;
# (NC)  $NVP{'nologp'} = cutnumber($ses_newu{'TOTAL'} / $totvisitors *100) if $totvisitors;
# (NC)  $NVP{'snap_noorder'} = 0 if ( $NVP{'snap_noorder'} < 0 );      

        # (NC)
        $NVP{'snap_log'} = $ses{'log'}; 
        $NVP{'logp'} = 0;
        $NVP{'logp'} = cutnumber($ses{'log'} / $totvisitors *100) if $totvisitors;
        $NVP{'snap_nolog'} = $ses{'nolog'};
        $NVP{'nologp'} = 0;
        $NVP{'nologp'} = cutnumber($ses{'nolog'} / $totvisitors *100) if $totvisitors;
        #$NVP{'snap_noorder'} = $ses{'nolog'};
        $NVP{'visleftp'}=0;
        $NVP{'visleftp'} =cutnumber ( $NVP{'snap_visleft'} / $totvisitors*100) if $totvisitors;
        # (NC)

        $NVP{'noorderp'} = 0;
        $NVP{'noorderp'} = cutnumber($NVP{'snap_noorder'} / $totvisitors *100) if $totvisitors;

        $NVP{'leftp'} = 0;
        $NVP{'leftp'} = cutnumber($NVP{'snap_left'} / $totvisitors *100) if $totvisitors;

        $NVP{'leftorderp'} = 0;
        $NVP{'leftorderp'} = cutnumber($NVP{'snap_leftorder'} / $totvisitors *100) if $totvisitors;

        $NVP{'checkoutp'} = 0;
        $NVP{'checkoutp'} = cutnumber($NVP{'snap_checkout'} / $totvisitors *100) if $totvisitors;

        $NVP{'newjoinp'} = 0;
        $NVP{'newjoinp'} = cutnumber($NVP{'snap_newjoin'} / $totvisitors *100) if $totvisitors;

	if ($totvisitors) {
                $con_rate = cutnumber($tot{'pageviews_raw'}/$totvisitors);
	} else {
		$con_rate = 'N/A';
	}
	$NVP{'snap_per_visitor'} = $con_rate;

        if ($totvisitors) {
                $con_rate = $tot{'orders_num'}/$totvisitors;
                $con_rate = substr($con_rate, 0, 6);
        } else {
                $con_rate = 'N/A';
        }
	
        $NVP{'snap_convert'} = $con_rate;

        #report
        $targ = "row_stats_report";
        $t = loadtemplate($targ, $skeletonpath);
        $t =~ s/<!--.*-->\n//s;

	$i = 0;
	foreach $pageurl (sort topreqsort keys %tot_req) {
		$i++;
                last if $i>$TOP25;
                $viewpercent = cutnumber($tot_req{$pageurl}/$tot{'request'} *100);
		@page = split(/\//, $pageurl);
                @objid = split(/\./, $page[$#page]);

                    if ($objid[0] eq 'home') {
                           $pagetitle = 'Homepage';
                    }
                    else{
                         $pagetitle = ucfirst($title{$objid[0]});
                    }
	   
            
            $pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

	    $REP{'NUM'} = $i;
            $REP{'TITLE'} = "$pagetitle<br>";
	    $REP{'URL'} = $pagerefurl;
	    $REP{'NUMBER'} = $tot_req{$pageurl};
	    $REP{'PERCENT'} = $viewpercent;

	    $NVP{'top25page'} .= permutebit($t, \%REP);
	}
	
	$i = 0;
	foreach $pageurl (sort leastreqsort keys %tot_req) {
		$i++;
		last if $i>$TOP25;
                $viewpercent = cutnumber($tot_req{$pageurl}/$tot{'request'} *100);
		@page = split(/\//, $pageurl);
                @objid = split(/\./, $page[$#page]);

                if ($objid[0] eq 'home') {
                       $pagetitle = 'Homepage';
                }
                else{
                       $pagetitle = ucfirst($title{$objid[0]});
                }

		$pagerefurl = "http://$site->{'siteserver'}$site->{'sitedomain'}$pageurl";

		$REP{'NUM'} = $i;
                $REP{'TITLE'} = "$pagetitle<br>";
                $REP{'URL'} = $pagerefurl;
                $REP{'NUMBER'} = $tot_req{$pageurl};
                $REP{'PERCENT'} = $viewpercent;   

                $NVP{'least25page'} .= permutebit($t, \%REP); 
	}

	$i = 0;
        $targ="row_stats_topbrowser";
        $tt = loadtemplate($targ, $skeletonpath);
        $tt =~ s/<!--.*-->\n//s;

	foreach $pageurl (sort topbrosort keys %ses_bro) {
		$i++;
		last if $i>$TOP25;

#   (NC)        $viewpercent = cutnumber($ses_bro{$pageurl}/$totvisitors *100) if ($totvisitors);
                $viewpercent = cutnumber($ses_bro{$pageurl}/$tot_bro *100) if ($tot_bro); #(NC)
                $BR{'NUM'} = $i;
                $BR{'BROW'} = $pageurl;
                $BR{'NUMBER'} = $ses_bro{$pageurl};
                $BR{'PERCENT'} = $viewpercent;

                $NVP{'top25browser'} .= permutebit($tt, \%BR);
	
}

         	$i = 0;
        foreach $pageurl (sort toprefsort keys %tot_ref) {
		$i++;
		last if $i>$TOP25;
                $viewpercent = cutnumber($tot_ref{$pageurl}/$tot{'refer'} *100);

                $REP{'NUM'} = $i;
                $REP{'TITLE'} = "";
                $REP{'URL'} = $pageurl; 
                $REP{'NUMBER'} = $tot_ref{$pageurl};
                $REP{'PERCENT'} = $viewpercent; 

		$NVP{'top25refer'} .= permutebit($t, \%REP);
	}

#order information
	$NVP{'order_num'} = $tot{'orders_num'};
	$NVP{'order_total'} = cutnumber ($tot{'orders_tot'}); 
        $NVP{'order_ship'} = cutnumber( $tot{'orders_shp'});
        $NVP{'order_tax'} = cutnumber( $tot{'orders_tax'});

        $targ="row_stats_topitem";
        $t = loadtemplate($targ, $skeletonpath);
        $t =~ s/<!--.*-->\n//s;
	$i = 0;
	$snap .= "&"; 

        foreach $lineobjid (sort toplineobjidsort keys %tot_lineobjid) {
           $i++;
           last if $i>$TOP25;
           my $lineobjid_sale = $lineobjid_price{$lineobjid}*$tot_lineobjid{$lineobjid};
           $lineobjidurl = "http://$site->{'siteserver'}$site->{'sitedomain'}/html/$site->{'om_locale_lang_def'}/$lineobjid\.shtml";
           $ITEM{'NUMBER'} = $i;
           $ITEM{'LINEOBJIDURL'} = $lineobjidurl;
           $ITEM{'LINEOBJID'} = $lineobjid;
           $ITEM{'TOT_LINEOBJID'} = $tot_lineobjid{$lineobjid};
           $ITEM{'PRICE'} = $lineobjid_price{$lineobjid};
           $ITEM{'SALE'} = cutnumber($lineobjid_sale);

           $NVP{'topitems'} .= permutebit ($t, \%ITEM);
        }

#Advertising Summary

	$targ="row_stats_ad";
	$t = loadtemplate($targ, $skeletonpath); 
        $t =~ s/<!--.*-->\n//s;
   #adbuy
	foreach $adbuy (sort adbuysort keys %ses_adb) {
		$con_rate = $ses_adb_pur{$adbuy} / $ses_adb{$adbuy} if $ses_adb{$adbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_adb{$adbuy} / $ses_adb_pur{$adbuy} if ($ses_adb_pur{$adbuy});
		$AD{'AD'} = $adbuy ;
		$AD{'VISITOR'} = $ses_adb{$adbuy};
		$AD{'ORDER'} = $ses_adb_pur{$adbuy};
		$AD{'TOTAL'} = $tot_adb{$adbuy};
		$AD{'AVESALE'} = $avesale;
		$AD{'CONVERSION'} = $con_rate;
		
		$NVP{'adbuy_row'} .= permutebit($t, \%AD);
	}

   #artwork
	foreach $artbuy (sort artsort keys %ses_art) {
		$con_rate = $ses_art_pur{$artbuy} / $ses_art{$artbuy} if $ses_art{$artbuy};
		$con_rate = substr($con_rate, 0, 6);
		my $avesale = $tot_art{$artbuy} / $ses_art_pur{$artbuy} if ($ses_art_pur{$artbuy});

		$AD{'AD'} = $artbuy;
                $AD{'VISITOR'} = $ses_art{$artbuy};
                $AD{'ORDER'} = $ses_art_pur{$artbuy};
                $AD{'TOTAL'} = $tot_art{$artbuy}; 
                $AD{'AVESALE'} = $avesale;
                $AD{'CONVERSION'} = $con_rate;
 
                $NVP{'artwork_row'} .= permutebit($t, \%AD);
	}
}

#write to the stats file
$targ = "os_allcommerce-runstats";
$l = loadtemplate($targ, $skeletonpath);
$l = permutebit($l, \%NVP);
open(STAT, ">$targetpath/$filedate.html");
print STAT $l;
close(STAT);

$snap = "$fromtime"."&"."$totime"."&"."$statuser";
$snap .= "\&$NVP{'snap_page'}";
$snap .= "\&$NVP{'snap_visitor'}";
$snap .= "\&$NVP{'order_num'}";
$snap .= "\&$NVP{'order_total'}";
$snap .= "\&$NVP{'topitems'}";
$snap .= "&"."$filedate";
print STDOUT $snap if ($snapflag);

$sth->finish if ( $sth );
$dbh->disconnect if ( $dbh );

sub cutnumber {
   my $temp =  shift @_;
   $temp += 0.005 ;
   my ($t1, $t2) = split /\./, $temp;
   $t2 .= "00";
   $t1 .= "\.".substr($t2, 0, 2);
   return $t1;
}

#sort rules
sub topreqsort  { $tot_req{$b} <=> $tot_req{$a}; }
sub leastreqsort { $tot_req{$a} <=> $tot_req{$b}; }
sub toprefsort { $tot_ref{$b} <=> $tot_ref{$a}; }
sub topbrosort { $ses_bro{$b} <=> $ses_bro{$a}; }
sub adbuysort { $ses_adb{$b} <=>  $ses_adb{$a}; }
sub artsort { $ses_art{$b} <=>  $ses_art{$a}; }
sub topobjidsort { $tot_objid{$b} <=>  $tot_objid{$a}; }
sub toplineobjidsort { $tot_lineobjid{$b} <=> $tot_lineobjid{$a}; }
#sub getgifname {};
1;
