#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# update_conf.pl
#    reads .conf files from the ./conf directory and 
#    creates a new site.pm file in that directory
#
# Version 28 TG 20000921
#    Only add ORACLE stuff for ORACLE database
# Version 27 JC 20000710
#    cut back on what gets quotemeta
# Version 26 JC 20000707
#    quotemeta functionality added
# Version 25 JC 20000608
#    updated header file that gets generated for site.pm
#    to more generic in the conf files that get put into
#    site.pm
# Version 24 JC 20000607
#    change directory for om_db.pm to lib from conf
# Version 23 JC 20000530
#    detect null entries in %conf and delete them
# Version 22 JC 20000513 
#    allow only [a-zA-Z].conf files to be processed
# Version 21 JC 20000511
#    strip ALL spaces before and after in processing values (lines 153 adn 154) 
# Version 20 JC 20000411
#    spaces in values, slash in @ for email, remove siteshortname
# Version 19 JC 20000402
#    modified so that conf files do not have quotes in them
# Version 18 JC 20000330
#    correction to connect statement
# Version 15 JC 20000330
#    put in default database driver as mysql
# Version 14 IB 20000327
#    corrected $relpath
# Version 13 JC 20000327
#    corrected a typo
# Version 12 JC 20000326
#    addition of om_db.pm values
# Version 11 JC 20000323
#    removed repetitive building of conf hash
# Version 10 IB 20000323
#    pure magic
# Version 9 JC 20000322
#    moved : for ssl_prefix out 
#    added command line argument to determine fsrelativeroot to take care 
#    of symlink with presents 
# Version 8 JC 20000320
#    corrected comment statements and put in #!/usr/bin/perl for site.pm
#    added : for ssl_prefix
# Version 7 IB 20000319
#    corrected a comment statement
# Version 6 JC 20000317
#    debuged version 5, took out tree.conf, and completed script
# Version 5 IB 20000315
#    changed file names
# Version 4 IB 20000302
#    removed -w
# Version 3 IB 20000301
#    updated copyright
# Version 2 IB 20000226
#    changed store to site
# Version 1 RF 991115
#    THIS IS AN INCOMPLETE SCRIPT, please edit store.pm manually
#       until we release 0.7.pre3 of the system.
#    created, reads *.conf files from ../conf/
#    writes store.pm in ../conf/
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

require "$relpath/lib/om_db.pm";

$db=om_db();

%conf=();        # null hash for configuration parameters

%database=();    # temporary holder for parameters from database.conf
%site=();        # temporary holder for parameters from site.conf
%machine=();     # temporary holder for parameters from machine.conf

opendir CONF, "$relpath/conf";
@allfiles = readdir CONF;
closedir CONF;

while (@allfiles) {
   $f = pop @allfiles;              # get a file from the list
   next if ( length( $f ) <= 5 );   # .conf has five characters
   next if ( $f !~ /\.conf$/ );     # it has to end in .conf
   next if ( $` =~ /[^a-zA-Z]/ );   # compare name for non-letters

   open( FH, "<$relpath/conf/$f" );       # read the file
   @lines=(<FH>);
   close(FH);

   $h={};                           # blank hashref
   while ( @lines ) {               # go through the lines
      $_ = pop @lines;
      next if ( $_ =~ /^#/ );          # skip comment lines
      chomp;
      next if ( $_ =~ /^$/ );             # skip blank lines
      ( $k,$v ) = split /\s/, $_, 2;
      $v =~ s/^\s+//g;
      $v =~ s/\s+$//g;
      $v =~ s/\@/\\\@/g if ( $v !~ /\\/ );
      $h->{$k} = $v;
   }                                # done reading conf data
   %site=%{$h} if ( $f eq "site.conf" );
   %machine=%{$h} if ( $f eq "machine.conf" );
   %database=%{$h} if ( $f eq "database.conf" );
   foreach $k (keys %{$h} ) { $conf{$k}=$h->{$k}; }
}                                   # done running through files
sort %conf;
## print to file site.pm

$ORACLE_HOME=$ARGV[0];
$ORACLE_SID=$ARGV[1];

open SITEPM, ">$relpath/conf/site.pm"; 

## print out #!/usr/bin/perl
print SITEPM "#!/usr/bin/perl \n\n";

## print out header info and warning not to edit this file anymore

print SITEPM "# This file was generated automatically by update_conf.pl from the 
# contents of <name>.conf where <name> is an alpha charactered named file.  Don't edit 
# this file, edit the <name>.conf files instead. \n\n"; 

if ($ORACLE_HOME) {
    print SITEPM "\$ENV{'ORACLE_HOME'}=\"$ORACLE_HOME\";\n";
}
if ($ORACLE_SID) {
    print SITEPM "\$ENV{'OARCLE_SID'}=\"$ORACLE_SID\";\n\n";
}

print SITEPM "sub setsite { 
	# returns a hashref
	my(\$h,\$m,\$c,\$sr,\$rr,\$dbc,\$dbkv);
	\$h={};

	## actually set environmental params now
	#	
	# For a detail information about the following fields,
	# please see the document DOCS/install.html
	\$h = { \n "; 

## Here is where the hash table gets printed.  Need to remove $sr.
## Also include fsrelativeroot
@quotemeta = ("sitename","currency_symbol");
@noquote = ("site_sku_numlen", "site_sku_sclen", "maxtopnav", "site_sku_totlen", "site_sku_prelen");
@singlequote =("site_database_serv", "sitecode", "ssl_prefix", "sitessldomain", "sitedomain", "siteserver", "sitesslserver");
print SITEPM "\tfsrelativeroot\t=> '$siteshortname',\n"; 
foreach $k ( @noquote )
	{  
          print SITEPM "\t$k\t=> $conf{$k}, \n" if $k; 
          delete $conf{$k}; 
        } 

foreach $k ( @singlequote )
        { 
          print SITEPM "\t$k\t=> \'$conf{$k}\', \n" if $k;
          delete $conf{$k}; 
        } 

foreach $k ( keys %conf )
	{ 
            if ( $k !~ /\$sr/ ) { 
               foreach $item (@quotemeta) {
                  $conf{$k} = quotemeta $conf{$k} if ($item eq $k);
               }
               print SITEPM "\t$k\t=> \"$conf{$k}\",\n" if $k; 
            }
        } 

## Done with hash table
print SITEPM "     }; # done with  \$h \n";

       ## \$m is the machine specific (file system) bits hashref
       #      it's date is used to construct \$h, then it's keys 
       #      and values get read into \$h before returning \$h
       # \n";

## Here is where $sr gets placed.
foreach $k ( keys %conf )
        { if ( $k =~ /\$sr/ ) { print SITEPM "       $k = \"$conf{$k}\";\n"; }}
print SITEPM "       \$rr = \"\$h->{\'fsrelativeroot\'}\";

       \$m = {
\t   fsrootweb	=> \$sr,
\t   fsrootcgi	=> \"\$sr\/\$rr\/cgi-bin\",
\t   fsrootlib	=> \"\$sr\/\$rr\/lib\",
\t   fsroothtml	=> \"\$sr\",
\t   fsrootcron	=> \"\$sr\/\$rr\/crons\",
\t   fsrootlog	=> \"\$sr\/\$rr\/logs\",
\t   fsrootdata	=> \"\$sr\/\$rr\/data\"
\t}; # end of defining file system parameters\n";

## Database format from db_om.pm 
$site_driver = $conf{'site_datadriver'};
if ($site_driver =~ /^$/ ) { $site_driver="mysql"; }
$db->{'db_site_config'}{$site_driver} =~ s/sid\=/sid\=$ORACLE_SID/;

print SITEPM "## Database format from db_om.pm 
\t \$h->{\'db_site_config\'}=\"$db->{'db_site_config'}{$site_driver}\";
\t \$h->{\'db_site_connect\'}=\"$db->{'db_site_config'}{$site_driver}\";  
\t## construct some deterministic composite elements
\t#
\t\$h->{\'siteurl\'}=\$h->{\'siteserver\'}\.\$h->{\'sitedomain\'};
\t\$h->{\'sitesslurl\'}=\$h->{\'sitesslserver\'}\.\$h->{\'sitessldomain\'};

\tforeach ( keys %{\$m} ) {
\t   \$h->{\$_}=\$m->{\$_} if ( ! defined(\$h->{\$_}) ); 
\t}

\treturn \$h;
} #\/setsite

1;\n"; 

close SITEPM;

# make sure it has the proper file permissions
chmod 0755, "$relpath/conf/site.pm"; 
	
1;
