#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# update_symlinks.pl
#  RF 981112 to update the symlinks from ./auto and ./man to html
#
# Version 0.7.11 JC 20000712
#    fixed ln statements
# Version 0.7.10 JC 20000427
#    moved multilingual support from generate_site to this script
# Version 0.7.9 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.8 JC 20000411
#    default to eng if no default lang
# Version 0.7.7 JC 20000405
#    shifted synlinks to $def_lang
# Version 0.7.6 JC 20000331
#    added $def_lang support
# Version 0.7.5 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.4 IB 20000302 
#    removed -w
# Version 0.7.3 IB 20000301
#    updated copyright
# Version 0.7.1 - IB 991112
#    updated absolute directory name, eliminated ARGV[0]
# Version 4 - JK 980617
#    added links for .html and .htm files in man
# Version 3 - RF 981203
#    added removal of not-current-generation pages
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/conf/site.pm";

## establish site-specific parameters
#
$site=setsite();

## get languages that will be used to generate symlinks
#
$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

@langs=split /,/, $site->{'om_locale_lang_act'};
$notlangs=$site->{'om_locale_lang_inact'}; # inactive languages
@langsgen=("$om_render_lang");  # include default language

if ( $notlangs ) {
   foreach $lang (@langs) {
      if ( $notlangs !~ $lang ) { @langsgen=("@langsgen","$lang"); }; # do not include languages that are inactive
   } # done checking for inactive languages
}  elsif ( @langs ) {
@langsgen=("@langsgen","@langs");
   }

foreach $language (@langsgen) {
$om_render_lang = $language;  

chdir "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/html/$om_render_lang";

## remove files older than 65 minutes from the directory, since
#     this script is invoked at the end of the site generation 
#     cycles...
#
# TBD...

`/bin/ln -sf ./auto/*.shtml .`;
`/bin/ln -sf ./man/*.shtml .`;
`/bin/ln -sf ./man/*.html .`;
`/bin/ln -sf ./man/*.htm .`;
}
1;
