#!/usr/bin/perl 
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#
# xml_order_export.pl
#
# Program to create xml order files for transfer to warehouse
#    arguments: storecode, orderid
#
# Version 0.9.4 JC 20000913
#    oracle integration
# Version 0.9.3 JC 20000712
#    fixed dbipwd
# Version 0.9.2 IB 20000601
#    added custom-lib.pm
# Version 0.9.1 IB 20000502
#    introduced $script variable
# Version 0.7.14 IB 20000411
#    $def_lang -> $om_render_lang
# Version 0.7.13 JC 20000411
#    default to eng if no default lang
# Version 0.7.12 JC 20000410
#    multilingual messages
# Version 0.7.11 JC 20000330
#    changed connect statement
# Version 0.7.10 JC 20000328
#    changed DBI perpare/connect
# Version 0.7.9 IB 20000327
#    corrected $relpath
# Version 0.7.8 IB 20000324
#    changed the way site.pm is obtained (multiple stores)
# Version 0.7.7 IB 20000301
#    updated copyright
# Version 0.7.6 IB 20000229
#    changed table to database
# Version 0.7.5 IB 20000226
#    changed store to site
# Version 0.7.4 IB 991217
#    eliminated ERLOG
# Version 0.7.3 IB 991115
#    correction of database cleanup
# Version 0.7.2 IB 991112
#    corrected the determining of the relpath
#    removed foreach
#    corrected $scriptname
# Version 0.7.1 IB 991111
#    calculation of a relative path
#    using of new global variables
# Version 5 RF 991012
#    added status of "xd" = xml dumped
#    changed to take HQUANT+SQUANT as quantity to ship
#    changed to ignore fulfillable or not
# Version 4 RF 990921
#    added objects for use in XML templates:
#       vsku_uc   - uppercase VSKU
#       lineid_uc - uppercase LINEID
#       oid_uc    - uppercase OID/ORDERID
# Version 3 RF 990921
#    readout of order creation date from timestamp
#    fixed retrieval of order line items
# Version 2 RF 990921
#    better logging of error conditions
#    fixed absence of store fsrelativeroot from template load path
# Version 1 RF 990921
#
#

## determine path from store (site) root and store db name...
#
#  note - this script runs at server root or in crons,
#    generally. Test these conditions first.
#  $0 - may not contain a full path, bootstrap with "pwd"
#
if ( $0 !~ /^\// ) {  # $0 is not an absolute path
   $s=`pwd`;          # hope pwd on path...
   chomp $s;
   $s.="/$0";
} else {              # $0 IS an absolute path
   $s=$0;
} # done determining if $0 is relative or absolute
@temp=split /\//, $s;

@rpath=@temp;

$scriptname = pop @rpath;
$scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

$rundir= join "/", @rpath;
chdir( $rundir );

$relpath="";
$siteshortname="";
## short circuit if $s does not contain crons 
if ( $s !~ /crons/ ) {
   $siteshortname = pop @rpath;
   $relpath=".";
   @temp=();
} # done testing for degenerate case

while ( @temp ) {
   $u = pop @temp;

   ## skip references to the present directory
   next if ( $u eq '.' );
   next if ( length($u)==0 );        # skip null directory levels

   $relpath.="/.." if ( $relpath);
   $relpath.="." if ( ! $relpath );

   if ( $u eq "crons" ) {
      $siteshortname = $temp[$#temp];
      @temp=();
   } # note - crons is off the site root, so the next up is site root

} # done constructing relative path to store root

$relpath .= "/../$siteshortname";

use DBI;
require "$relpath/conf/site.pm";
require "$relpath/lib/xml.pm";
require "$relpath/lib/custom-lib.pm";

## establish site-specific parameters
#
$site=setsite();

$om_render_lang="$site->{'om_locale_lang_def'}";
$om_render_lang="eng" if ( ! $om_render_lang );

%script_msg=();
$script=( split /\./, $scriptname )[0];
&message;

chdir "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/crons/";

@orders=@ARGV[1..$#ARGV];         # slice containing orders

## establish connection in
#
$s = "$site->{'db_site_config'}"; 
$dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});

$targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/transit/export/orders";

## Load templates for output XML
#
%template=();

$template{'BOD36'}=xml_loadtemplate( "BOD36", $absroot ); 
$template{'ADDRESS'}=xml_loadtemplate( "ADDRESS", $absroot ); 
$template{'DATETIME'}=xml_loadtemplate( "DATETIME", $absroot ); 
$template{'PICKLINE'}=xml_loadtemplate( "PICKLINE", $absroot ); 

########### general flow
#
# validation of input args ( can be order list )
#
# for each:
#    test for existing output file
#    test for complete export capability
#    generate event for transit file generation or failure
#    update status of order to "sent to warehouse"
# 
#

foreach $order ( @ orders ) {

   $curord={};                        # current order hashref
   $blocks={};                        # blocks being assembled
   $order_is_shippable=1;             # default assume order is pickable

   ## pull order header information
   #
   $t=$order;
   $t =~ s/\-//;
   $q="SELECT \* from orders where OID LIKE '$t%' ";
   $sth = $dbh->prepare( $q );
   $sth->execute;
#   $num=$sth->rows;
#   if ( $num >= 1 ) {      # one order found...
#      $curord = $sth->fetchrow_hashref;
   while ( $curord = $sth->fetchrow_hashref(NAME_lc) ) {
      $c = ( keys %{ $curord } );
   } else {                # this is an error - wrong record count for order
      ## TBD - error handling 
   } # done pulling order header data
   $sth->finish;

   ## uppercased order pseudo-fields
   #
   $curord->{'oid_uc'}=uc( $curord->{'OID'} );

   ## construct timestamps
   #
   $h={};
   @time=localtime( $curord->{'CDATE'} );
   $time[5]+=1900;                   # add century offset
   $time[4]++;                       # month 0 is Jan, change so 1 is Jan
   @ntime=$script_msg{1};
   foreach $i ( 0..5 ) {
      $h->{$ntime[$i]}=$time[$i];
   } # done pushing time elements up
   $h->{'subsecond'}="1000";
   $h->{'qualifier'}="CREATION";
   $blocks->{'datetime:CREATION'}=xml_permute( $template{'DATETIME'}->{'skel'}, $h );

   $h={};
   @time=localtime( time() );
   $time[5]+=1900;                   # add century offset
   $time[4]++;                       # month 0 is Jan, change so 1 is Jan
   @ntime=$script_msg{1};
   foreach $i ( 0..5 ) {
      $h->{$ntime[$i]}=$time[$i];
   } # done pushing time elements up
   $h->{'subsecond'}="1000";
   $h->{'qualifier'}="DOCUMENT";
   $blocks->{'datetime:DOCUMENT'}=xml_permute( $template{'DATETIME'}->{'skel'}, $h );

   ## construct shipto and customer
   #
   $blocks->{'partner:ShipTo'}="";
   $blocks->{'partner:Customer'}="";
   $q="SELECT \* from addresses where ADDRID='$curord->{'SHIPTO'}'" ;
   $sth = $dbh->prepare( $q );
   $sth->execute;
#   $num=$sth->rows;
#   if ( $num > 0 ) {    # got at least one address, use one
#      $h = $sth->fetchrow_hashref;
   while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {
      $h->{'partnertype'}="ShipTo";
      $blocks->{'partner:ShipTo'}.=xml_permute( $template{'ADDRESS'}->{'skel'}, $h );
      $h->{'partnertype'}="Customer";
      $blocks->{'partner:Customer'}.=xml_permute( $template{'ADDRESS'}->{'skel'}, $h );
   } else {             # need error trapping - bad address
      ## TBD - error trap and handling for zero address records
   } # done pulling shipping address data

   ## construct picklines
   #
   $blocks->{'picklines'}="";
   $q="SELECT \* from ocart where OID='$order' ";
   $sth = $dbh->prepare( $q );
   $sth->execute;
#   $num=$sth->rows;
#   foreach ( 1..$num ) {
      ## pull each line item, check for SQUANT>0 as an error
      # 
#      $h = $sth->fetchrow_hashref;
   while ( $h = $sth->fetchrow_hashref(NAME_lc) ) {
      ## ignore fillability as of v5 until better inventory data available 
      #
      # commented $order_is_shippable=0 if ( $h->{'SQUANT'}>0 );   # FLAG as unfillable order

      ## construct VSKU pseudo-field and populate uppercased pseudofields
      #
      $h->{'VSKU'}=$h->{'SKU'}.$h->{'VCODE'};
      $h->{'vsku_uc'}=uc( $h->{'VSKU'} );
      $h->{'lineid_uc'}=uc( $h->{'LINEID'} );

      ## TBD - retrieve sku title and variant encoding expansion
      # 
      $h->{'TITLE'}="product name unavailable";
      $q2="SELECT TITLE from things where SKU='$h->{'SKU'}'";
      $stt=$dbh->prepare( $q2 );
      $stt->execute;
#      if ( $stt->rows > 0 ) {
#         $h->{'TITLE'}=$stt->fetchrow_hashref->{'TITLE'};
      if ( $h->{'TITLE'}=$stt->fetchrow_hashref->{'TITLE'} ) {
      } # done loading title, if possible


      ## construct XML pickline
      #
      $blocks->{'picklines'}.=xml_permute( $template{'PICKLINE'}->{'skel'}, $h );
   } # /done assembling rows

   ## last minute additions to curord
   #  
   $curord->{'revisionid'}="$time[0]";
   $curord->{'siteshortname'}="MyHome";

   $output=$template{'BOD36'}->{'skel'};

   $c = ( keys %{ $curord } );

   $output=xml_permute( $output, $blocks );
   $output=xml_permute( $output, $curord );

   ## test to see if generating a free-standing order file or not
   #
   if ( $order_is_shippable ) {                # free standing orderfile
      open( FH, ">$targetpath/$t.xml" );
      print FH "$output \n";
      close( FH );
      ## TBD - generate event and change order status flag
      #
      
      $q = "UPDATE orders SET STATUS='xd' where OID='$curord->{'OID'}' and STATUS!='xd' ";
      $dbh->do( $q ); 

   } else {                                    # unshippable just yet
      ## TBD - generate export failure event
      #
   } # done with deciding if order is fulfillable or not

} # done examining each order requested
#

sub message {
   # check the language that is being used and open appropriate
   # message language file or get messages below

   if ($def_lang eq "eng" ) {
      $script_msg{1}="('second','minute','hour','day_om','month','year')";
   } else {
      open( EM, "<$relpath/bits/$om_render_lang/system/$script.messages");
      while(<EM>) {
         chomp;
         next if /^\#/;
         next if (! $_);
         @temp=split /\t/;
         $script_msg{$temp[0]}=$temp[1];
      }
      close(EM);
   }
}

1;
