#!/usr/bin/perl
#
####################################################################
#
# CyberCash CashRegister interface for OpenSales AllCommernce 1.0.1
#
#  (c)2000 Guardian Digital, Inc.
#
#  This module creates a temporary file containing the order 
#  number, encrypted credit card number and expiration date
#  to be read in by the CashRegister scripts.
#
####################################################################
#
# + 09/06/00 - Nick DeClario <nick@guardiandigital.com>
#   - Added Credit Card Name to temp file
#   - Retreieved Blowfish key from merchant config
#
# + 08/31/00 - Nick DeClario <nick@guardiandigital.com>
#   - Added 56byte (448bit) Blowfish encryption for credit card number
#
# + 08/29/00 - Nick DeClario <nick@guardiandigital.com>
#   - Created
#
#######################################
#
sub run_cybercash {
  use Crypt::Blowfish;

  ### Retrieve the key from the config file
  #
  $relpath = &getRelPath();
  $configFile = $relpath . "/conf/merchant_conf";
  my $key;  

  open (CONFIG, "< $configFile") || die ("$configFile not found!");
  @lines = <CONFIG>; 
    foreach $line (@lines) {
       if (substr($line, 0, 12) eq "BLOWFISH_KEY") {
          $key = $line;
       }

    }
  close (CONFIG);

  $key =~ s/BLOWFISH_KEY //s;
  $key =~ s/\=//g;
  $key =~ s/^ //g;
  chomp $key;

  my ($in, $oh, $cctemp, $cipher, $encryptedCCNum);
  $in = $_[0];
  $oh = {};

  ### Get base SSL URL from OpenSales config 
  #
  require "$relpath/conf/site.pm";
  $h = &setsite();
  $url = "https://$h->{'sitesslserver'}$h->{'sitessldomain'}/cgi-bin/mck-cgi/directpaycredit.cgi";

  $invoice_id = $in->{'orderid'};
  $ccnum = $in->{'CCID'} if ($in->{'CCID'});
  $exp_date = sprintf("%s/%s",$in->{'CCEXPMO'},$in->{'CCEXPYR'});
  $cost=$in->{'ordtotal'};
  $ccName = $in->{'CCNAME'};
  chomp $ccName;  

  $invoice_id =~ tr/+/ /;

  $invoice_id =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

  $invoice_id =~ s/\r/ /g;
  $invoice_id =~ s/\n/ /g;

  chomp $ccnum;

  #break credit card number into two 8byte strings for encryption
  $cctemp = substr ($ccnum, 0, 8);
  my $cipher = new Crypt::Blowfish $key;

  my $encryptedCCNum1 = $cipher->encrypt($cctemp);
  $cctemp = substr ($ccnum, 7, 8);
  my $encryptedCCNum2 = $cipher->encrypt($cctemp);

  open (DATA, "> $relpath/transit/temp/mck.tmp");
  print DATA "$invoice_id\n$encryptedCCNum1\n$encryptedCCNum2\n$exp_date\n$ccName\n";
  close (DATA);

  chmod (0400,"$relpath/transit/temp/mck.tmp");

  print "Location:  $url/\n\n";

}

sub getRelPath {

  if ( $0 !~ /^\// ) {  # $0 is not an absolute path
     $s=`pwd`;          # hope pwd on path...
     chomp $s;
     $s.="/$0";
     } else {              # $0 IS an absolute path
     $s=$0;
  } # done determining if $0 is relative or absolute
  @temp=split /\//, $s;

  @rpath=@temp;
  $scriptname = pop @rpath;
  $scriptname = ( split /\?/, $scriptname )[0];   # ignore GET stuff

  $rundir= join "/", @rpath;
  #chdir( $rundir );

  $relpath="";
  $siteshortname="";
  ## short circuit if $s does not contain cgi-bin
  if ( $s !~ /cgi-bin/ ) {
     $siteshortname = pop @rpath;
     $relpath=".";
     @temp=();
  } # done testing for degenerate case

  while ( @temp ) {
     $u = pop @temp;

     ## skip references to the present directory
     next if ( $u eq '.' );
     next if ( length($u)==0 );        # skip null directory levels

     $relpath.="/.." if ( $relpath);
     $relpath.="." if ( ! $relpath );

     if ( $u eq "cgi-bin" ) {
        $siteshortname = $temp[$#temp];
        @temp=();
     } # note - cgi-bin is off the site root, so the next up is site root
  }
  $relpath .= "/../$siteshortname";

  return $relpath;

}


return false;
