#!/usr/bin/perl 
############################################################################
#
# generate_triggers.pl
#
# Program to generate temporary data files
#    to feed into various general pulldowns
#
# Version 6 TG 20000926
#   Fixed mutiple variant title problem
# Version 5 JC 20000913
#   oracle intergration
# Version 4 JC 20000828
#   fixed evaluate as bug
# Version 3 JC 20000822
#   bux fix for master variant display
# Version 2 JC 20000818
#   added removal of old pages
# Version 1 JC 20000814
#   creation based on version 0.9.20
#   of generate site
#     

sub generate_triggers {

my %h=();
my %templates=();   # hash of templates by short name
my %objid=();        # hash of hashrefs to sku data
my %variants=();    # hash of hashrefs to variants, keyed on VSKU
my %linkup=();      # hash on vsku/sku of arrays( of hashrefs ) topbar 
my %linkdown=();    # hash on vsku/sku of arrays( of hashrefs ) on-page
my %columns=();     # hash of hashrefs of columns per template
my %axiscodes=();   # hash of hashrefs of plaintext per encoding
my %defcells=();    # default cells for different objects
my %extantvars=();  # extant variants per master sku
my %stock=();       # hash of stock values
my %skus=();
## setup default cells by object
#
my %defcells=();
$defcells{'variants'}="cell_140_master_vv";
$defcells{'subcat'}="cell_123_search_80_sc";
$defcells{'masterprod'}="cell_123_search_80_mp";

if ( ! $_[0] ) {
    exit;
}

## get languages that will be used to generate template
#
my @langs=split /,/, $site->{'om_locale_lang_act'};
my $notlangs=$site->{'om_locale_lang_inact'}; # inactive languages
my @langsgen=("$om_render_lang");  # include default language 

if ( $notlangs ) {
   foreach my $lang (@langs) {
      if ( $notlangs !~ $lang ) { push @langsgen,$lang; }; # do not include languages that are inactive
   } # done checking for inactive languages
}  elsif ( @langs ) {
      push @langsgen,@langs;
   } 

my @interfaces=split /,/, $site->{'interfaces'};
if ( ! @interfaces ) { @interfaces="html"; }

my $q="SELECT objid FROM things WHERE visibility!='110000-A' AND objclass='134-A' OR visibility!='110000-A' AND objclass='136-A'";
$sth = $dbh->prepare( $q );
$sth->execute;  
while ( my $remove = $sth->fetchrow ) { 
   foreach my $lang ( @langsgen ) {
      foreach my $interface ( @interfaces ) {
         if ( -e "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/$interface/$lang/$remove.shtml" ) {
            `rm $site->{'fsroothtml'}/$site->{'fsrelativeroot'}/$interface/$lang/$remove.shtml`;   
            `rm $site->{'fsroothtml'}/$site->{'fsrelativeroot'}/$interface/$lang/auto/$remove.shtml`;  
         }
      }
   }
}

my $q2="SELECT * from things WHERE visibility='110000-A' ";
$sth = $dbh->prepare( $q2 );
$sth->execute;
#my $num=$sth->rows;
#foreach ( 1..$num ) {
#   $h=$sth->fetchrow_hashref;
while ( $h=$sth->fetchrow_hashref(NAME_lc) ) {

   ## backconvert from mysql-ish text and make html safe  
   foreach $k ( keys %{$h} ) {
      next if ( "title,brief|pagestyle" !~ /$k/ );
      $h->{$k}=string_exmysql($h->{$k});     
      next if ( "pagestyle" eq $k );       # do not html-ize this
      $h->{$k}=string_tohtml($h->{$k}); 
   }
      $objid{$h->{'objid'}}=$h;
      $variants{$h->{'objid'}}=$h;
}

$q3 = " SELECT * from prices WHERE status='15000-A' ";
$stp=$dbh->prepare($q3);
$stp->execute;
#my $nump=$stp->rows;
#for (1..$nump) {
#my   $p = $stp->fetchrow_hashref;
while ( my $p = $stp->fetchrow_hashref(NAME_lc) ) {
   next if ( $p->{'unitct'}!=1 );            # only need unit prices...
   next if ( $p->{'groupid'} );              # for anyone
   if ( (! $objid{$p->{'objid'}}->{'unitpr'}) || ($p->{'unitpr'} < $objid{$p->{'objid'}}->{'unitpr'}) ) {
      $objid{$p->{'objid'}}->{'unitpr'}=$p->{'unitpr'};
      $variants{$p->{'objid'}}->{'unitpr'}=$p->{'unitpr'};
   }
}

#load extracopy fields, as applicable
my $q3 = "SELECT objid,fieldname,body,language,status FROM extraprop ";
$stg = $dbh->prepare( $q3 );
$stg->execute;
#my $necq = $stg->rows;
#foreach ( 1..$necq ) {
#   ( $k,$c,$b,$l,$st ) = $stg->fetchrow_array;
while ( ( $k,$c,$b,$l,$st ) = $stg->fetchrow_array ) {
   $l="$om_render_lang" if ( ! $l ); # set to default language if none is given
my   $b=string_exmysql($b);
my   $b=string_tohtml($b);
    $variants{$k}->{$l}->{$c}=$b if ( ! $variants{$k}->{$l}->{$c} && ( $st eq "15000-A" || $st eq "" ) );
    $variants{$k}->{$l}->{'status'}=$st if ( ! $variants{$k}->{$l}->{'status'} && $st eq "15000-A" ); 
} # done running through extracopy fields 

my $q="SELECT objid,lstock from inventory";
$sts = $dbh->prepare( $q );
$sts->execute;
#$num=$sts->rows;

#foreach (1..$num) {
#      $h=$sts->fetchrow_hashref;
while ( $h=$sts->fetchrow_hashref(NAME_lc) ) {      
      $stock{$h->{'objid'}}=$h->{'lstock'} if ($h->{'lstock'});
       
} # done running through inventory...



## load all linkage data
#
#     %linkage{objid} for objid=obj1
#     %toplinkags{objid} for objid=obj2
#
$q="SELECT \* FROM linkage WHERE status='15000-A' ORDER BY cdate";
$sth = $dbh->prepare( $q );
$sth->execute;
#$num=$sth->rows;
#foreach ( 1..$num ) {
#   $h=$sth->fetchrow_hashref;
while ( $h=$sth->fetchrow_hashref(NAME_lc) ) {
   my $s1=$h->{'obj1'};
   my $s2=$h->{'obj2'};
   $h->{'cellstyle'}=string_exmysql($h->{'cellstyle'});

   ## push up topbar stuff, where this objid would be in OBJ2
   push @{$linkup{$s2}}, $h;

   ## push up on-page links, where this objid would be in OBJ1
   push @{$linkdown{$s1}}, $h;

} # done loading links



## sort linkage lists by weight
#
#  TBD - RF get back to this 990901
#

my $q2="SELECT * from things WHERE visibility='110000-A' "; 
#$q2="SELECT * from things ";    
$sth = $dbh->prepare( $q2 );
$sth->execute;
#$num=$sth->rows;
#foreach ( 1..$num ) {
#   $h=$sth->fetchrow_hashref;
while ( $h=$sth->fetchrow_hashref(NAME_lc) ) {
   ## backconvert from mysql-ish text and make html safe
   foreach $k ( keys %{$h} ) {
      next if ( "title,brief|pagestyle" !~ /$k/ );
      $h->{$k}=string_exmysql($h->{$k});
      next if ( "pagestyle" eq $k );       # do not html-ize this
      $h->{$k}=string_tohtml($h->{$k});
   }
   # done with loading all information on this skus to hash...
   if ($h->{'ref_from'} ne $h->{'objid'}) {
      $extantvars{$h->{'ref_from'}}.="\t" if $extantvars{$h->{'ref_from'}};
      $extantvars{$h->{'ref_from'}}.=$h->{'objid'};
   }

my $thevariant="";                        # plaintext axis values

   if (( $variants{$h->{'ref_from'}}->{'axisnames'} ) and ( $h->{'ref_from'} ne $h->{'objid'})) {
      my @ab = ( split /:/, $variants{$h->{'ref_from'}}->{'axisnames'} );
      my @tmp_code_array = ();  ## 2D array to hold variant codes  x  y  each variant sku is x, values in variant are y
      my @tmp_decode_array = (); ## 2D array that matches codes
      my $tmp_yax_num  = 0;
      my $tmp_xax_num  = 0;
      my @maxarray = ();  ## array of maximum num  of variants in y direction
      ## Loop for languages - fill in with default language if no other language is present
#      foreach $language (@langsgen) {
        foreach $a (@ab){  
           ## if variant does not exist for a specifc lanugae use default
           if ( ($variants{$a}->{$om_render_lang}->{'encoding'}) and ( ! $variants{$a}->{$language}->{'encoding'} ) ) {
              $variants{$a}->{$language}->{'encoding'} = $variants{$a}->{$om_render_lang}->{'encoding'};
           }
           #must add the two iterations together so that k = VCODE
           #must check to see if there's more thatn one
           if ( ( ! $axiscodes{$a} ) and ( $variants{$a}->{$language}->{'encoding'} )) {   # the decoding not loaded yet
              my @tomap = split /(\n|\r|\f)+/, $variants{$a}->{$language}->{'encoding'};  # tomap now contains ind sku's complete 1D variance table
              $tmp_yax_num = 0;
              foreach ( @tomap ) {         # get the actual code/decodes
                 (my $k,my $v) = split /:/, $_ ;    # split that table up into code/decode
                 next if ( ! $k );            # if the cde is nothing get out of here
                 my $v2 = $v;
                 $v2 =~ tr/ //d ;             # if decode is nothing insert warning
                 next if ( ! $v2 );
                                        #have to build new temp table here
                 $tmp_code_array[$tmp_xax_num][$tmp_yax_num]=$k;
                 $tmp_decode_array[$tmp_xax_num][$tmp_yax_num]=$v;
                 $tmp_yax_num++;
              }
              $maxarray[$tmp_xax_num]=$tmp_yax_num;
           } # done if
           $tmp_xax_num++;
           my @current_val_array = ();
           my $maxnum = 1;
           for $n (0.. $tmp_xax_num)    {$current_val_array[$n]=0;  ## init array to all zeros
           if ($maxarray[$n]) {$maxnum = $maxnum * $maxarray[$n];} ## find out maximum number of possibilites
        } # done foreach $a

        for $i (0 .. $maxnum)  ## create all possibilities of multidimensional variants
           {
           my $fin_code = "";
           my $fin_decode ="";
           for $n (0.. $tmp_xax_num)                                                                                                             {
              if (($fin_decode) and ($tmp_decode_array[$n][$current_val_array[$n]])) {$fin_decode.=", ";} ### if in next pass there exists a decode at a coma    

              $fin_decode .= $tmp_decode_array[$n][$current_val_array[$n]];
              if (($fin_code) and ($tmp_code_array[$n][$current_val_array[$n]])) {$fin_code.=':';} ## if in next pass there exists a code add a :
              $fin_code .= $tmp_code_array[$n][$current_val_array[$n]];
           }

        $current_val_array[$#current_val_array]++;

my        $carryover=0;
        for ($i = $#current_val_array; $i >= 0; $i--){
                $current_val_array[$i] += $carryover;
                $carryover = 0;
                if ($current_val_array[$i] >= $maxarray[$i]){
                        $current_val_array[$i] = 0;
                        $carryover = 1;
                }
        }
           
#           @current_val_array = &increment_array(\@current_val_array, \@maxarray); ## passing references to arrays and incrementing them
           $axiscodes{$h->{'objid'}}->{$fin_code}=$fin_decode;
        }
        $thevariant.=", " if ( $thevariant );
        $thevariant.=$axiscodes{$h->{'objid'}}->{$h->{'vcode'}};
#      } # done foreach
   } # done loading plaintext by axisname
   ## decode the axis
   #
   if ( $thevariant ) {
      $h->{'variant_plaintext'}=$thevariant;
   } # done adding plaintext translation of axis values

  $varprice=$variants{$h->{'objid'}}->{'unitpr'}; #socheata
  $variants{$h->{'objid'}}=$h;
  $variants{$h->{'objid'}}->{'unitpr'}=$varprice;
} # done loading things data to hash
}
## check for variant-less skus
# 
foreach $k ( keys %objid ) {
   next if ($variants{$k});
   next if ($objid{$k}->{'ref_from'} ne $objid{$k});
   $variants{$k}=$objid{$k};
#   $variants{$k}->{'vcode'}="";
} # done writing null variants into variants table

## fill in variant information
#
foreach my $k ( keys %variants ) {

   if ( $variants{$k}->{'objid'} && $variants{$k}->{'ref_from'} && $variants{$k}->{'objid'} ne  $variants{$k}->{'ref_from'} ) {
#      $variants{$k}->{'title'}=$variants{$variants{$k}->{'ref_from'}}->{'title'}." ".$variants{$k}->{'variant_plaintext'};
       $variants{$k}->{'title'}=$variants{$variants{$k}->{'ref_from'}}->{'title'};
      $variants{$k}->{'brief'}=$variants{$variants{$k}->{'ref_from'}}->{'brief'};
   }
}     

## page generation
#
my @gen;
push @gen, $_[0];

if ( $variants{$_[0]}->{'ref_from'} && $variants{$_[0]}->{'ref_from'} ne $_[0] ) {
   push @gen, $variants{$_[0]}->{'ref_from'}
} 

foreach my $s ( @gen ) {

#foreach $s ( sort keys %variants ) {

next if ( $variants{$s}->{'visibility'} ne "110000-A" );

   foreach $interface (@interfaces) {

      ## run through each language to generate pages   
      #
      foreach $language (@langsgen) {
      my $om_render_lang = $language;

      my $targetpath="$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/$interface/$om_render_lang/auto";

      my $lng=$variants{$s}->{'language'};
      $lng="$site->{'om_locale_lang_def'}" if ( ! $lng );

         if ( $lng eq $om_render_lang ) {
#            $sku = $variants{$s}->{'msku'};
#            $vcode = $variants{$s}->{'vcode'};
            my $targ = "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/$interface/$om_render_lang/auto/$sku.shtml";
            ## check the template is loaded
            #
            my $tn=$variants{$s}->{'pagestyle'};
            ## Is this page supposed to be generated at all
            # 
   

            ## skip generation if this is nondisplay or out of stock and not persistent
            #
            ## TBD - reinstate when trust status flags
            #next if ( $variants{$s}->{'status'} eq "nd" );

            ## set a default pagestyle if blank
            $tn="page_product_master" if ( ! $tn );     # temp - want all pgs


            ## override page template if the site has variants and it is
            #     a variant page
            #
            $tn="page_product_master_vv" if ( ( $variants{$s}->{'vcode'} ) and ( $tn =~ /master$/ )); 

            ## give title and descript from master to variants
            #
            if ( $variants{$s}->{'vcode'} ) {
               $z=$variants{$s}->{'ref_from'};
#               $variants{$s}->{'title'}=$variants{$z}->{'title'};
               $variants{$s}->{$om_render_lang}->{'descript'}=$variants{$z}->{$om_render_lang}->{'descript'}; 
               $variants{$s}->{$om_render_lang}->{'alttitle'}=$variants{$z}->{$om_render_lang}->{'alttitle'};
            }
            
      $templates{$tn}=localize_template( $tn, $om_render_lang, $interface, 0 );

      my @temp2 = ( $templates{$tn} =~ m/__column_(\w+?)__/gs );
      my $y={};
      while (@temp2) {
         ( $k,$v ) = ( split /_/, shift @temp2 )[0..1];
         $y->{$k}=$v if ( ! exists ( $y->{$k} ) );      
         $y->{$k}=$v if ( $v > $y->{$k} );
      } # done running through columns
      $columns{$tn} = $y;     

            my $thispage = $templates{$tn};
            my %nccol=%{$columns{$tn}};  # number of columns per cell width

            ######
            ### Start Generation of on-page Cells
            ######

            ## load down linkage data of relevance
            #
            my @links = @{$linkdown{$s}};

            @links = sortlinks(@links); 
            ## override default linkage for master products - show only variants
            #
            if ( $s eq $variants{$s}->{'ref_from'} ) {
               @links=();
               ## create link to own variants 
               #
               my @temp = split /\t/, $extantvars{$s};
               foreach $k ( @temp ) {
                  $h=();
                  $h->{'obj1'}=$s;             # from this sku
#                  $h->{'obj2'}="$variants{$k}->{'msku'}"."$variants{$k}->{'vcode'}";             # to it's variant
                  $h->{'obj2'}=$k;
#                  $h->{'title'}=$variants{$k}->{'title'};     # needs a title
                  $h->{'linkwt_use'}=$variants{$k}->{'weight_use'}; # relative weight
                  $h->{'cellstyle'}=$defcells{'variants'};

                  push @links, $h;
               } # done pushing phony links up
            } # done with load

         ## generate cells from this...
         #     go through links in order, construct %expand{colwidth}
         # 
         my %expand=(); # holds expanded strings for column substitution
         my %cellct=(); # count of linkage cells per cellwidth
         foreach $i ( 0..$#links ) {
            $q="";
            my $u=$links[$i]->{'cellstyle'} ;
            next if ( ! $variants{$links[$i]->{'obj2'}}->{'visibility'} );    
          #next if ( length($u)<8 );

            my $x= ( split /_/, $u )[1];

      $templates{$u}=localize_template( $u, $om_render_lang, $interface, 0 );

      @temp2 = ( $templates{$u} =~ m/__column_(\w+?)__/gs );

      my $f={};
      while (@temp2) {
         ( my $k,my $v ) = ( split /_/, shift @temp2 )[0..1];
         $f->{$k}=$v if ( ! exists ( $f->{$k} ) );  
         $f->{$k}=$v if ( $v > $f->{$k} );
      } # done running through columns
      $columns{$tn} = $f;     

            if ( $expand{$x} ) {
               $cellct{$x}++;
               $expand{$x}.="\n <!-- START_ELEMENT_$cellct{$x} --> \n";
            } else {
               $cellct{$x}=1;
               $expand{$x}="<!-- start of column -->\n";
               $expand{$x}.="\n <!-- START_ELEMENT_$cellct{$x} --> \n";
               }
            ## add subcell handling...
            #
   
            ## overwrite default fields with manual entries
            #     just TITLE for now...
            %h=%{$variants{$links[$i]->{'obj2'}}};
            $h{'title'}=$variants{$links[$i]->{'objid'}}->{$om_render_lang}->{'alttitle'} if ( $variants{$links[$i]->{'objid'}}->{$om_render_lang}->{'alttitle'} && $variants{$links[$i]->{'objid'}}->{$om_render_lang}->{'status'} eq "15000-A");
            my $image=$s."_80x80.jpg";
            my $image2=$s.".gif";      
            if ( -e "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$om_render_lang/products/80/$image" || -e "$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$om_render_lang/duotone/$image2" ) {
               $h->{'image'}=$s;
               $h{'image'}=$s; 
            } else {
               $h->{'image'}="default";
               $h{'image'}="default";
         

            }
            $h{'unitpr'}=$objid{$links[$i]->{'obj2'}}->{'unitpr'}; #socheata add in

            my $tempvar = $h{'objid'};
            if (($stock{$tempvar} <=  0) and ($variants{$tempvar}->{'behavior'} =~ /(os|dp)/ )) {
              $h{'cartlink'} =  "<img src=/images/$om_render_lang/buttons/button_outofstock.gif> ";
            } else {
              $tempvar = $h{'objid'};
              $h{'cartlink'} = "<input type=\"image\" src=\"/images/$om_render_lang/buttons/button_purchase.gif\" BORDER=0 alt=\"add to cart\">";
            }          

            $q=permutebit( $templates{$u}, \%h );

            $expand{$x}.=$q;      
         } # done constructing gross column substitution units
         ## now construct bits in case multiple columns per width
         #

         ## split up column substitution strings as needed
         #
         foreach my $w ( keys %nccol ) {
            my $n = $nccol{$w};
            if ( $n == 1 ) {
               $k="column_$w"."_1";
               $variants{$s}->{$k}=$expand{$w};
               ## geometry addin
               $expand{$w} =~ s/__c__/$n/g; 
 
               my @temp_str = split /\n/, $expand{$w};
               my $temp_out = "";
               my $temp_r=0; 
               foreach (@temp_str) {
                  if ($_ =~ /START_ELEMENT/) { $temp_r++; }
                  $_ =~ s/__r__/$temp_r/g;
                  $temp_out .= $_ . "\n";
               }
               $expand{$w} = $temp_out;         

               $variants{$s}->{$k}=$expand{$w};
            } else {
               $u = $cellct{$w}/$nccol{$w};
               if ( $u>int($u) ) {
                  $u=int($u)+1;
               }
               # use $u cells per column of this type
               ## assemble columns
               #
               foreach my $k ( 1..$n ) {
                  my $min = (( $k - 1 ) * $u ) + 1 ;
                  my $max = ($k * $u) + 1;
                  next if ( $min > $cellct{$w} );
                  $min='<!-- START_ELEMENT_'.$min.' -->';
                  $max='<!-- START_ELEMENT_'.$max.' -->';
                  if ( $expand{$w} =~ /$min/ ) { # there is something to cut
                     $q = ( split /$min/, $expand{$w} )[1];
                  } else { 
                     $q = $expand{$w};
                  } # done cutting too early bits
                  if ( $k < $n ) {
                     $q = ( split( /$max/, $q ) )[0]
                     if ( $max < $cellct{$w} ) ;
                  } # done cutting too late bits
                  $q =~ s/__c__/$k/g;# 
                                                       
                  my $temp_r = 0;
 
                  my @temp_str = split /\n/, $q;
                  my $temp_out = "";
 
                  foreach (@temp_str) {
                     if ($_ =~ /START_ELEMENT/) { $temp_r++; }
                     $_ =~ s/__r__/$temp_r/g;
                     $temp_out .= $_ . "\n";
                  }
                  $q = $temp_out;       


                  my $ky="column_$w"."_$k";
                  $variants{$s}->{$ky}=$q;
               } # done assembling column bits
            } # done with split or not split branches    
         } # done splitting up columns for substitution into template 

         ## topnav construction -- load "up" linkage data of relevance
         #
         my @links=@{$linkup{$s}};   

         ## push link for main sku if the vcode nonnull
         #
         if ( $variants{$s}->{'vcode'} ) {
            $h={};
            $h->{'obj1'}=$variants{$s}->{'ref_from'};
            $h->{'obj2'}=$s;
            $h->{'title'}=$variants{$variants{$s}->{'ref_from'}}->{'title'};
            $h->{'linkwt_use'}=$variants{$variants{$s}->{'ref_from'}}->{'weight_use'};
            push @links, $h;
            push @links, @{$linkup{$z}};
         } # done forging linkage
         
         ## create sorted list on weight
         #
         my @links = sortlinks( @links );

         ## construct topnav 
         #
         my %knownlinks=();                              # blank hash for links
         $q="";

         my $x = $#links;

         $x=($site->{'maxtopnav'} - 1) if ( $x > ( $site->{'maxtopnav'} - 1) );
         foreach $i ( 0..$x ) {
            # check for duplicate links
            next if ( $knownlinks{$links[$i]->{'obj1'}} );
            next if ( ! $variants{$links[$i]->{'obj1'}}->{'title'} );
            $knownlinks{$links[$i]->{'obj1'}}="here";  # note "been here, done that"
            $q.=" - " if ( $q );
            $q.='&nbsp;<A HREF="/html/'.$om_render_lang.'/'.$links[$i]->{'obj1'}.'.shtml">';   
            $q.=$variants{$links[$i]->{'obj1'}}->{'title'};
            $q.='</A>&nbsp;';
            $replacetitle=$variants{$links[$i]->{'obj1'}}->{'title'};
         } # done constructing topnav
  
 
         ## insert topnav if real, otherwise insert include of default
         if ( length($q)>2 ) {
	    $variants{$s}->{'jump_to_nav'}=$q;
         } else {
            $variants{$s}->{'jump_to_nav'} = "<a href=\"/html/$om_render_lang/home.shtml\">Home Page</a>";
         } # done with jump_to_nav  

         ## computed field-like things...
         #
         #
         if ( -e "/$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$om_render_lang/headers/$s.gif" ) {
            $variants{$s}->{'headergif'} = "$s.gif";
         } else {
            $variants{$s}->{'headergif'} = "genericheader.gif";
         } #/headers   

         $image=$s."_80x80.jpg";
	
         if ( -e "/$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$om_render_lang/products/80/$image" || -e "/$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/images/$om_render_lang/duotone/$image" ) {
         $variants{$s}->{'image'} = $s;
         } else {
            $variants{$s}->{'image'} = "default";
         }      
         if (($stock{$s} < 1) and ($variants{$s}->{'behavior'} =~ /(os|dp)/ )) {
              $variants{$s}->{'cartlink'} =  "<img src=/images/$om_render_lang/buttons/button_outofstock.gif> ";
         } else {
              $variants{$s}->{'cartlink'} = "<input type=\"image\" src=\"/images/$om_render_lang/buttons/button_purchase.gif\" BORDER=0 alt=\"add to cart\">";

         }         
         ## backgroundimage
         #
         $variants{$s}->{'site_background'} = $site->{'site_background'};
         $variants{$s}->{'sitename'} = $site->{'sitename'};
         ## permute the file and dump to the filesystem
         #
         $t=permutebit( $thispage, $variants{$s} ); 
         ## permute any variables from site hash
         #

         $t=permutebit( $t, $site );
         $t=permuteclean( $t  );

         if ( $interface eq "html" || $interface eq "palm" ) {
            $filetype="shtml";
         }  else {
            $filetype=$interface;
         }

         if ($catch{$s} ne "yes" ) {
            open( FH, ">$targetpath/$s.$filetype" );  
            print FH $t;
            close(FH);  
            $tot_page++;
         }
	
         if ( $interface eq "html" ) {


            my $root=$s;

            ##socheata
            #change later?

            my @temp=split /\t/, $extantvars{$root};


            if (($#temp==0)  && (length($s) > 8) ) {
    
               my $space="";
               my $tobereplace="<A HREF=\"/html/$om_render_lang/$s.shtml\">$replacetitle</A>";

               $t=~ s/$tobereplace/$space/g;

               $t=~ s/<\!--replace-->/$variants{$s}->{'jump_to_nav'}/g;

               $t=~ s/<a href=\"\/html\/$om_render_lang\/$s.shtml\"><IMG SRC=\"\/images\/$om_render_lang\/buttons\/view_button.gif/<IMG SRC=\"\/images\/$om_render_lang\/general\/clear.gif/g;
               open (FH2, ">$targetpath/$s.$filetype");
               print FH2 $t;
               close(FH2);
               $tot_page++;
               $catch{$root}="yes";
               }

           }

        }
 
      }
   }
}

`$site->{'fsroothtml'}/$site->{'fsrelativeroot'}/crons/update_symlinks.pl`;

$stp->finish if ( $stp );
$sts->finish if ( $sts );
$sth->finish if ( $sth );
$stv->finish if ( $stv );
$ste->finish if ( $ste );
$stg->finish if ( $stg );

}

1;
