#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################
#  variant.pm
#
#   NOTE:  $relpath MUST BE SET BY THE SCRIPT THAT IS REQUIRING THIS LIBRARY!
# Version 1.0.12 NC 20000915
#    put \" \" arround the value of variant select option
# Version 1.0.11 JC 20000913
#    oracle intergration
# Version 1.0.10 TG 20000801
#    Fixed SQL statements
# Version 1.0.9 TG 20000710
#    Added isProductMaster and isVariantProduct, added master_fields to
#    get_fields_and_title
# Version 1.0.8 TG 20000628
#    Added support for sel_ pulldown support
# Version 1.0.7 TG 20000620
#    Added support for variant pulldown text with '-' in the text
# Version 1.0.6 TG 20000615
#    Added more variant support
# Version 1.0.5 TG 20000614
#    Added support for variant products
# Version 1.0.4 TG 20000614
#    Added support for variant products
# Version 1.0.3 IB 20000608
#    removed -w from the first line
#    commented out the statement: $debug = 1
# Version 1.0.1 TG 
#    added getVariant
# Version 1.0.0 TG created the first version

use DBI;
require "$relpath/lib/formtools.pm";

$debug = 0;
if ($debug) {
    open VAR_ERLOG, ">/tmp/variant";
}

###################################################################################################
# PURPOSE     
#   To retreive the title for objects, as well as any additional requested fields.
#   A simple query to the things table is only alright if the item is not a variant. 
#   If it is a variant, it gets the title based on the master title and the body of 
#   the extraprop table for the ref_from objid
#
# INPUT  
#   $fields - additional fields to include in the selection from things
#   $where_clause - additional statements to be added to the whereclause for the query from things
#   $objid - the primary key of things, to base the query on
#   $dbh - current active database handler
#   $master_fields - optional, extra fields to get from the master record
#
# OUTPUT
#   $results - a hash, to be referenced by the fields queried in things
#
# NOTES
#   Only 1 row will ever be returned, as objid is the primary key of things.
###################################################################################################
sub get_fields_and_title {
    my ($fields) = $_[0];
    my ($where_clause) = $_[1];
    my ($objid) = $_[2];
    my ($dbh) = $_[3];
    my ($master_fields) = $_[4];

    my ($q, $sth, @results_arr, @master_arr, @extraprop_arr, $results, $extraprop, $master_row, @rows, $row, $title);
    my ($multiple_var, $vcode_indx, @vcodes, @variants);

    if ($debug) {
	print VAR_ERLOG "IN get_fields_and_title with $objid\n";
    }
    # Added axisnames, ref_from and vcode to populate title of variant
    $q="SELECT title, axisnames, ref_from, vcode";

    # if there are additional fields append them
    if ($fields) {
	$q .= ", $fields";
    }

    # Since objid is the primary key, only one row will be returned from this query
    $q .= " FROM things WHERE objid='$objid'";

    # if there are additional where clause cases, append them
    if ($where_clause) {
	$q .= " $where_clause";
    }

    if ($debug) {
	print VAR_ERLOG "Select from things with --> $q\n";
    } 

    $numRows = &query($q, $dbh, \@results_arr);

    # we know that there is only one row returned, so get it!
    $results = $results_arr[0];
    $results->{'numrows'} = $numRows;

    if ($debug > 1) {
	foreach $key (keys %{$results}) {
	    print VAR_ERLOG "\$results->{$key} = $results->{'$key'}\n";
	}
    }
    if ( $numRows) {
	if ($results->{'ref_from'} ne $objid && $results->{'ref_from'} ne "") {
	    if ($debug) {
		print VAR_ERLOG "Need to set objid variant title!\n";
		print VAR_ERLOG "Need to set objid variant title!\n";
	    }
	    
	    # get the master title
	    $q="SELECT title, axisnames ";
	    if ($master_fields ne "") {
		$q .= ", $master_fields ";
	    }
	    $q .= "FROM things WHERE objid = \'$results->{'ref_from'}\'";
	    $numRows = &query($q, $dbh, \@master_arr);

	    if ($debug) {
		print VAR_ERLOG "get_fields_and_title -- Q -> $q\n";
	    }
	    
	    # only will have one returned result
	    $master_row = $master_arr[0];
	    if ($debug) {
		print VAR_ERLOG "$master_row->{'title'}, $master_row->{'axisnames'}, $master_row->{'brief'}\n";
	    }

	    if ($numRows) {
		$results->{'title'} = $master_row->{'title'};

		# if asked for master_fields, then get them
		if ($master_fields ne "") {
		    @fields = split (/\,/,$master_fields);
		    foreach $fld (@fields) {
			$results->{$fld} = $master_row->{$fld};
			if ($debug) {
			    print VAR_ERLOG "Just set extrafield $fld to $results->{$fld}\n";
			}
		    }
		}

		$multiple_var = $master_row->{'axisnames'};
		if ($debug) {
		    print VAR_ERLOG "Title is $results->{'title'}, MULTIPLE_VAR IS $multiple_var\n";
		}
		if ($multiple_var =~ /:/) {
		    if ($debug) {
			print VAR_ERLOG "$results->{'objid'} has multiple variants! $results->{'ref_from'}\n";
			print VAR_ERLOG "get_fields_and_title: $results->{'objid'} has multiple variants! $results->{'ref_from'}\n";
		    }

		    @variants = split /:/, $multiple_var;
		    @vcodes = split /:/, $results->{'vcode'};
		    if ($debug) {
			print VAR_ERLOG "Splitting on : for \$results->{'vcode'} --> $results->{'vcode'}\n";
			print VAR_ERLOG "vcodes are @vcodes!\n";
		    }
		}
		else {
		    $variants[0] = $multiple_var;
		    $vcodes[0] = $results->{'vcode'};
		}

		# append the title for each variant!
		$vcode_indx = 0;
		$v_codes = $vcodes[$vcode_indx];
#		@codes = split /:/,$v_codes;

		$results->{'title'} .= " - ";
		foreach $variant (@variants) {

		    if ($debug) {
			print VAR_ERLOG "get_fields_and_title: VARIANT: $variant \n";
			print VAR_ERLOG "VCODES ARE: \$v_codes -> $v_codes\n";
		    }
		    
		    # get the body to determine the text to take for th title
		    $q="SELECT body FROM extraprop WHERE objid = \'$variant\' AND fieldname = 'encoding'";
		    if ($debug) {
			print VAR_ERLOG "Extraprop query --> $q\n";
			print VAR_ERLOG "get_fields_and_title: Extraprop query --> $q\n";
		    }
		    
		    $numRows = &query($q, $dbh, \@extraprop_arr);
		    # only have one result returned
		    
		    $extraprop = $extraprop_arr[0];
		    
		    if ($numRows) {
			if ($debug) {
			    print VAR_ERLOG "Body is $extraprop->{'body'}\n";
			    print VAR_ERLOG "get_fields_and_title: Body is $extraprop->{'body'}\n";
			}
			@rows = split (/\n/,$extraprop->{'body'});
			$while_indx = 0;
			$done = 0;
			while ((!$done) && ($while_indx <= $#rows)) {
			    $row = $rows[$while_indx++];
			    # get rid of CR!
			    $row =~ s/\r//g;
			    # Use @val to take into account colon in the text of the pulldown
			    ($code, @val) = split(/:/,$row);       
			    if ($debug) {
				print VAR_ERLOG "Length of val is $#val!\n";
			    }
			    if ($#val == 0) {
				$val = $val[0];
			    }
			    else {
				$val = join ":", @val;
			    }

			    # look for the vcode in the enumerate list
			    if ($code eq $vcodes[$vcode_indx]) {
				$results->{'title'} .=  $val . " ";
				if ($debug) {
				    print VAR_ERLOG "Title is .... $results->{'title'} for variant!\n";
				    print VAR_ERLOG "get_fields_and_title: Title is .... $results->{'title'} for variant!\n";
				    }
				$done = 1;
			    }
			}
		    }
		    $vcode_indx++;
		}
	    }  # end of populate title for variant
	}
	$results->{'title'} = string_exdbi( $results->{'title'} );
    }
    return ($results);
}

###################################################################################################
# PURPOSE     
#   To retreive the objid and titles for all variants associated with a master objid.
#
# INPUT  
#   $master_objid - the master's objid.  The item you want to find the variants for
#   $dbh - current active database handler
#   $variants - pointer to an array, used as output.  Must be passed in as \@arrayName
#
# OUTPUT
#   $variants - a two dimensional array, which will consist of the variant
#               name ($variant[<indx>][0]) and title ($variant[<indx>[1]), where
#               indx is 0 --> number of variants - 1.
#
# NOTES
#   Only 1 row will ever be returned, as objid is the primary key of things.
###################################################################################################
sub getVariants {
    my ($master_objid, $dbh, $variants) = @_;
   
    my ($q, $sth, $res, @res_arr, $numRows);

    $q = "SELECT objid FROM things WHERE ref_from = \"$master_objid\" AND objid <> \"$master_objid\"";
    if ($debug) {
	print VAR_ERLOG "getVariants:  query --> $q\n";
    }

    $numRows = &query($q, $dbh, \@res_arr);

    for ($i = 0; $i < $numRows; $i++) {
	$res = $res_arr[$i];
	$$variants[$i][0] = $res->{'objid'};
	$titles = &get_fields_and_title("","",$$variants[$i][0], $dbh);
	$$variants[$i][1] = $titles->{'title'};
    }
	
   if ($debug > 1) {
	print VAR_ERLOG "getVariants: Number of rows are $numRows\n";
	for ($i =0; $i <= $numRows; $i++) {
	    print VAR_ERLOG "getVariants: Variants are $$variants[$i][0] --> $$variants[$i][1]\n";
	}
    }

    return $numRows;
}

sub getExistingVariants {
    my ($master_objid, $fields, $dbh, $variants) = @_;
   
    my ($q, $sth, $res, @res_arr, $numRows);

    $q = "SELECT ";
    $q .= $fields;
    $q .= " FROM things WHERE ref_from = \'$master_objid\' AND objid <> \'$master_objid\' order by vcode";

    if ($debug) {
	print VAR_ERLOG "getVariants:  query --> $q\n";
    }

    $numRows = &query($q, $dbh, \@res_arr);
    if ($debug) {
	print VAR_ERLOG "There are $numRows variants for $master_objid!\n";
    }
    for ($i = 0; $i < $numRows; $i++) {
	$res = $res_arr[$i];
	$titles = &get_fields_and_title("","",$res->{'objid'}, $dbh);
	$res->{'title'} = $titles->{'title'};

	# set the pulldowns for the row
	@pulldowns = ('sel_status|2','sel_visibility|2');
	foreach ( @pulldowns) {
	    ( $k,$v) = split /\|/, $_;
	    open(FH, "<$selskeletonpath/$k.sel");
	    @raw=<FH>;
	    chomp @raw;
	    close(FH);
	    if ($k =~ /sel_/) {
		$k =~ s/sel_//gms;
		$k = uc($k);
		if ($debug) {
		    print VAR_ERLOG "New pulldown name is $k\n";
		}
	    }
	    $kt=$k."_SELECT";
	    $k = lc($k);
	    $NVP{$kt}= gen_select( $res->{$k},$v,@raw );
	    $res->{$kt} = $NVP{$kt};
	    print VAR_ERLOG "JUST MADE \$NVP{$kt} --> $NVP{$kt} with \$res->{$k} --> $res->{$k}\n";
	}

	# set the ini, pre and inp for the row
	@temp = keys %{$res};
	while ( @temp ) {
	    $k = shift @temp;
	    $v = string_exdbi($res->{$k});
	    $kp=$k."_pre";
	    $kn=$k."_ini";
	    $ki=$k."_inp";
	    $res->{$k}=$v;
	    $res->{$kp}=$v;
	    $res->{$kn}=$v;
	    $res->{$ki}=$v;
	} # done permuting fieldnames


	# save the hash of the row
	$$variants[$i] = $res;
   }

   if ($debug) {
	print VAR_ERLOG "getVariants: Number of rows are $numRows\n";
	for ($i = 0; $i < $numRows; $i++) {
	    $res = $$variants[$i];
	    foreach $key (keys %res) {
		print VAR_ERLOG "$key --> $res->{$key}\n";
	    }
	}
		
    }

    return $numRows;
}


###################################################################################################
# PURPOSE     
#   To create a dynamic pull down for variants.  The pulldown is based on the array of variant
#   information passed into this routine.
#
# INPUT  
#   $base - 
#   $numVar - The number of variants to include in the pulldown list
#   @variants - Array of variants, details of datastructure in OUTPUT section of getVariants procedure
#               above.
#
# OUTPUT
#   $options - The HTML <OPTION ...> statements to be put within a SELECT HTML statement
#
# NOTES
#
###################################################################################################
sub gen_variant_sel {
    my ($base, $numVar, @variants) = @_;
    my ($options, $v, $d);

    $options="<OPTION VALUE=\"\">Not Selected";
    $d=1;
    for ($i = 0; $i < $numVar; $i++) {
	$options.= "<OPTION VALUE=\"$variants[$i][0]\" ";  

	if (( $base ) and ( $d ) and ( length($base)>=$l )) {
	    if ( "$base" eq "$variants[$i][1]" ) {
		$x.="SELECTED $variants[$i][1] = $base";
		$d=0;
	    }
	}
	$options.="> $variants[$i][1] \n";

	if ($debug) {
	    print VAR_ERLOG "gen_variant_sel: $options\n";
	}
   }
   return $options;
}




###################################################################################################
# PURPOSE     
#   To submit a query to the $dbh, returning results if there are any
#
# INPUT  
#   $q - query to submit
#   $dbh - active open database handler
#   $p_results - A pointer to an array to return the results back in.
#
# OUTPUT
#   $p_results -  An array of hashes, each index consisting on the simple has with the keys being
#                 the fields requested in the query
#   $rows - The number of rows returned
#
# NOTES
#
###################################################################################################
sub query {
    my ($q, $dbh, $p_results) = @_;

    my ($sth, $rows, $results, $i);

    $sth=$dbh->prepare($q);
    $sth->execute; 
#    $rows = $sth->rows;   
#    if ($rows <= 0) {
#	$sth->finish;
#	return $rows;
#    }
    # get the row if it is a select statement, otherwise we do not need the results
    $rows=0;
    $i=0;
    if (($q =~ /SELECT/) || ($q =~ /select/)){
        # get ALL rows returned
#	for ($i = 0; $i < $rows; $i++) {
#	    $$p_results[$i] = $sth->fetchrow_hashref(NAME_lc);
        while ( $$p_results[$i] = $sth->fetchrow_hashref(NAME_lc) ) {
           $rows++;
           $i++
	}
	$sth->finish;
    } else {
        while ( $rfh = $sth->fetchrow_hashref(NAME_lc) ) {	
           $rows++;
        } 
    }
    return $rows;
}


############################################################################
# PURPOSE     Determines if the product is a master product.
#
# INPUT       $objid - object to determine if the object is a master product
#             $dbh - live database handler
#
# OUTPUT      1 if it is master product, and set's the master_objid in the
#                  form
#             0 if it is not a master product
#
# NOTES      
############################################################################
sub isProductMaster {
   my ($objid) = $_[0];
   my ($dbh) = $_[1];
   
   my ($q, $numRows, @res, $h);
   
   $q = "SELECT ref_from FROM things WHERE objid = '$objid'";
   $numRows = &query($q, $dbh, \@res);
   
   $h = $res[0];
   # if the ref_from equals the objid, it is a master product
   if ($h->{'ref_from'} eq $objid) {
      $NVP{'master_objid'} = $objid;
      return 1;
   }
   
   return 0;
}

############################################################################
# PURPOSE     Determines if the product is a variant product
#
# INPUT       $objid - object to determine if the object is a variant product
#             $dbh - live database handler
#
# OUTPUT      objid of the master if it is a variant product
#             nothing if it is not a variant product
#
# NOTES   
############################################################################
sub isVariantProduct {
   my ($objid) = $_[0];
   my ($dbh) = $_[1];
   
   my ($q, $numRows, @res, $h);
   
   $q = "SELECT ref_from FROM things WHERE objid = '$objid'";
   $numRows = &query($q, $dbh, \@res);
   
   $h = $res[0];
   if ($debug) {
       print VAR_ERLOG "isVariantProduct\nRef_from is $h->{'ref_from'}, objid is $objid!\n";
   }
   # if the ref_from equals the objid, it is a master product
   if (($h->{'ref_from'} ne '') && ($h->{'ref_from'} ne $objid)) {
       return $h->{'ref_from'};  #return master objid
   }
}


1;
