#!/usr/bin/perl
############################################################################
#    Copyright (C) 1999-2000 Zelerate, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
############################################################################

####
# welcome-lib.pm
#    lib functions for welcome.cgi
#    LCF 20000828 
#
####
####
# Version 7 JC 20001004
#       fixed secure sid bug
# Version 6 JC  20001003
#       secure sid
# Version 5 LCF 20000926
#	made wel_validate_new_pass check for 5 characters, not 4
# Version 4 LCF 20000925
#   Made login case-insensitive (again)
# Version 3 LCF 20000919
#	Fix a lingering rows() for Oracle
#	Removed "bad char" filter in hints.
# Version 2 JC  20000915
#   oracle integration
# Version 1 LCF 20000901
#	created
#
####

#use strict;

# Set the site hashref and the %FORM hash, return them.
sub wel_set_site_data {

	## read site conf from setsite
	#my $site=&setsite(); 1.1 code
	$site;
	&setsite();

	## read input form and cookie data
	#
	#my ($form_ref) = &ReadParse2; 1.1 code
	%FORM;
	&ReadParse();
	
	
	return ($site, \%FORM);
}


sub wel_connect_db {
	my $site = shift;
	
	# Connect to DB
	my $q="$site->{'db_site_config'}";
	my $dbh = DBI->connect($site->{'db_site_connect'}, $site->{'site_dbiuser'}, $site->{'site_dbipwd'});
	
	return $dbh;
}

sub wel_get_lang_interface {
	my 	($FORM, $dbh, $site) = @_;

        $FORM->{'sid'}=&os_unhash(pack(H16,$FORM->{'sid'}), $site->{'hash_key'}) if ( $FORM->{'sid'} );

	my ($om_render_lang, $interface);
	if ($FORM->{'sid'}) {
		my $q="SELECT language,interface from sessions where sid=$FORM->{'sid'} ";
		my $sts = $dbh->prepare( $q );
		$sts->execute;
		my $h=$sts->fetchrow_hashref(NAME_lc); 
		$om_render_lang=$h->{'language'};
		$interface=$h->{'interface'};
                $sts->finish;
	}
	
	# If not found, look in $site
	$om_render_lang=$site->{'om_locale_lang_def'} unless $om_render_lang;
	# Total failure, default to english
	$om_render_lang="eng" unless $om_render_lang;
	
	# Default to html if no interface
	$interface="html" unless $interface;

	return ($om_render_lang, $interface);
}

sub wel_mode {
	my $mode = 'login';
	return $mode;
}

sub wel_account_ok_check {
	my 	$FORM = shift;
	
	if ( exists($FORM->{'new.email_ok_inp'}) ) {  # checkbox submitted
	   # carry state forward
	   $FORM->{'new.email_ok_pre'}=$FORM->{'new.email_ok_inp'};

	} elsif ( exists($FORM->{'new.email_ok_pre'}) ) {  # checkbox previously sub
	   $FORM->{'new.email_ok_inp'}=$FORM->{'new.email_ok_pre'};

	} else {                                    # no evidence of checkbox inp
	   $FORM->{'new.email_ok_inp'}="y";
	   $FORM->{'new.email_ok_pre'}="y";
	} #/done with checkbox permutations

	$FORM->{'new.email_ok_state'}="CHECKED" if ( $FORM->{'new.email_ok_inp'} eq "y" );

	return %{$FORM};
}

sub wel_page_targ {
	my ($site, $FORM, $scriptname, $om_render_lang) = @_;
        $sid_encrypt=unpack(H16, &os_hash($FORM->{'sid'}, $site->{'hash_key'}));
	if ( $ENV{'HTTP_REFERER'} ) {   # site referer if

    	# check that refered from inside site and not welcome
    	if (( $ENV{'HTTP_REFERER'} =~ /($site->{'sitedomain'}|$site->{'sitessldomain'})/ ) 
    	  and ( $ENV{'HTTP_REFERER'} !~ /$scriptname/ )) {
    	  $FORM->{'pg_targ'}=$ENV{'HTTP_REFERER'};	
	    } else { 
	      ## referer is outside site or this script, default to home
	      my $u=$site->{'siteserver'}.$site->{'sitedomain'};
	      $FORM->{'pg_targ'}="http://$u/html/$om_render_lang/home.shtml";
	    } #/done looking at plausibility of using the referer as targ

	} else {                                         # default target
	   my $u=$site->{'sitesslserver'}.$site->{'sitessldomain'};
	   $u="$site->{'ssl_prefix'}://$u/cgi-bin/om/ssl/checkout.cgi";
	   $FORM->{'pg_targ'}="$u?sid=$sid_encrypt";
	} #/done checking over page to use on outbound

	return %{$FORM};
}

sub wel_sort_form_input {
	my $FORM = shift;
		
	my $new = {};               # new customer data
	my $old = {};               # existing customer data

	foreach my $key ( keys %{$FORM} ) {
	   next if ( $key !~ /^(new|old)/ );

	   ## (don't) kill bad characters
	   #
	   my $value = $FORM->{$key};

	   #if ($key =~ /hint/) { 
       #  $value=~ tr/[]*&#^%$~+=\/\\<>\'\"//d; 
	   #} else {
       #  $value=~ tr/ []*&?#^%$!~+=\/\\<>,\'\"//d ;
	   #}

	   if ( $key =~ /^new/ ) {
    	  $key =~ s/^new\.// ;
    	  $new->{$key} = $value;

	   } else {
    	  $key =~ s/^old\.// ;
    	  $old->{$key} = $value;
       } #/done with new vs. old
	} #/done loading information to new/old pair

	# Put account data in 'acct' --  in lowercase!
	$new->{'acct'} = lc($new->{'email_inp'});
	$old->{'acct'} = lc($old->{'email_inp'});
	
	return ($new, $old);
}


sub wel_validate_old_account {
	my $old = shift;
	my $error = shift;
	my %FORM = @_;
	
	# return not ok, but no error if no login attempt (account field is null)
	return (0, 0) unless (length($old->{'acct'}) > 0);
	
	# Account field not null, see if it's too short
	if (length($old->{'acct'}) < 8) {
		# Set the account too short error.
		$FORM{'old.email_err'}=$error->{2};
		return (0, 1, %FORM);
	}
	
	# Account field ok.
	return (1, 0, %FORM);
}


sub wel_check_passwd_error {
	my $old = shift;
	my $error = shift;
	my %FORM = @_;

	
	# If the old email was blank...
	unless ( $FORM{'old.email_err'} ) {
		# And the password was not, set an error
		if (length($old->{'passwd_inp'}) > 0 )  {
			$FORM{'old.passwd_err'} = $error->{3};
			return (1, %FORM);
		} else {
			# Otherwise, just return.
			return (0, %FORM);
		}
	} 
	
	# If the old email was not blank...	
	if ( length($old->{'passwd_inp'}) > 4 ) { # minimalist passwd
    	$FORM{'old.passwd_err'} = $error->{3};
    } else {   # short passwd or blank field
        $FORM{'old.passwd_err'} = $error->{1};
    }

	return (1, %FORM);

}


sub wel_validate_old_pass {
	my $old = shift;
	my $error = shift;
	my %FORM = @_;

	# See if the password is long enough
	if (length($old->{'passwd_inp'}) < 5) {
		$FORM{'old.passwd_err'} = $error->{1};
		return (0, 1, %FORM);
	}

	return (1, 0, %FORM);
}


## customerlogin ( sid, username, passwd ): errstr;
#
sub wel_customerlogin {
   my ( $sid,$uname,$pwd,$site,$dbh,$script_msg ) = @_;

   my $q="SELECT objid,uname,passwd from contacts where ";
   $q.=" uname='$uname' and passwd='$pwd' and objtype='39003-A' ";
   my $sth = $dbh->prepare( $q );        # need to add password unhash
   my $rv;
   $sth->execute();

   my $goahead=0;
   my $howmany = 1;
   my $row = $sth->fetchrow_hashref(NAME_lc);
   
   if ($sth->fetchrow_hashref()) {
   	$howmany = 2;
   }
   $sth->finish();
  
   $sid=&os_unhash(pack(H16,$sid), $site->{'hash_key'}) if ($sid && length($sid) > 11);
 
   if ( ($howmany == 1) && ($row->{passwd} eq $pwd)) {
         my $now = time();
 
         ## valid session, now update customer timestamp 
         #
         $q="UPDATE contacts SET mdate=$now ";
         $q.= " where objid='$row->{objid}' ";
#         $dbh->do( $q );
         $sth = $dbh->prepare( $q );
         $sth->execute;
         ## now update session record
         #
         $q="UPDATE sessions SET custid='$row->{objid}', mdate=$now ";
         $q.=" where sid='$sid' "; 
#         $dbh->do( $q );
         $sth = $dbh->prepare( $q );
         $sth->execute;
   } else {
         ## too many or too few records, do not allow
         $rv=$script_msg->{19};
         ## now update session record
         #
		 my $now = time();
         $q="UPDATE sessions SET mdate = $now where sid='$sid' "; 
#         $dbh->do( $q );
         $sth = $dbh->prepare( $q );
         $sth->execute;
   } # done evaluating whether or not the customer in system

   $sth->finish;

   return $rv;
} #/customerlogin 


# Wrapper for validateemail. Needs to return 'a', 'b', 'c', or nothing
# a = nothing to validate
# b = invalid format (for email, no @)
# c = invalid format. Too. Hey, I didn't design it, ok.
sub wel_validate_acct {
	my $acct = shift;
	
	my $u=&validateemail($acct);
	
	return $u;
}


sub wel_set_new_acct_err {
	my $u = shift;
	my $script_msg = shift;
	my $new = shift;
	
	my $err_msg;
	if ( $new->{'acct'} ) {   # there was an input

      ## if this is not a correct email address
      #  what is the correct error to display
 
      if ( $u eq "a" ) {
         # Error - No email address entered!
         return (0, 1, $script_msg->{21});

      } elsif ( $u eq "b" ) {
         # Error - Wrong email address format, need \@
         return (0, 1, $script_msg->{22});
 
      } elsif ( $u eq "c" ) {
         # Error - Invalid email address!
         return (0, 1, $script_msg->{20});
 
      }

	} #/checking for any input

	return (0, 0, '');
}

sub wel_validate_new_pass {
	my ($new, $FORM, $script_msg) = @_;
	my $newok = 1;
	my $newerr = 0;
	
	
	if ( $new->{'passwd1_inp'} eq $new->{'passwd2_inp'} ) {
		# they match 
         if ( length( $new->{'passwd1_inp'} ) < 5 ) {
        	# message variation on lengths
            if ( ! $new->{'passwd1_inp'} ) {   # null condition
               $FORM->{'new.passwd_err'}=$script_msg->{6};
        
            } else {                         # just too short
               $FORM->{'new.passwd_err'}= $script_msg->{7};
            } #/message selection on length
   
            $newerr=1;
            $newok=0; 
         }
	} else {                       # passwords don't match
         $FORM->{'new.passwd_err'}=$script_msg->{8};
         $newerr=1;
         $newok=0;
    } #/done checking passwords for new user  
	
	return ($newok, $newerr, %{$FORM});
}

sub wel_unique_acct {
	my ($new, $dbh, $FORM, $script_msg) = @_;
	
	$new->{'acct'}=~ tr/A-Z/a-z/;
	my $q="SELECT objid from contacts where ";
	$q .=qq| uname='$new->{acct}' and objtype='39003-A'|;
	my $sth = $dbh->prepare( $q );
	$sth->execute();
	my $row = $sth->fetchrow;
	$sth->finish();
	
	if ($row) {
		$FORM->{'new.email_err'} = $script_msg->{5};
		return (0, 1, %{$FORM});
	}
	
	return (1,0,%{$FORM});
}


sub wel_create_acct_event {
	my ($site, $dbh, $new) = @_;
	my $now = time();
	
    if ( $site->{'site_datadriver'} =~ /Oracle/ ) {

        my $p= "INSERT into levents (event,series,dtable,dvalue,cdate) ";
        $p.=" values(levents_event.nextval,'$site->{'series'}','contacts',"; 
        $p.=" '$new->{'acct'}\t Create 39003-A', '$now') ";
        $stp=$dbh->prepare($p);
        $stp->execute;
        $stp->finish;    

        $q="SELECT levents_event.currval FROM DUAL";
        $stp=$dbh->prepare($q);
        $stp->execute;
        $event=$stp->fetchrow;
        $stp->finish; 

        my $e ="SELECT series from levents where dtable='contacts' and ";
        $e.=" dvalue='$new->{'acct'}\t Create 39003-A' and cdate='$now' ";
        $ste=$dbh->prepare($e);
        $ste->execute;
        my $series=$ste->fetchrow;
        $series =~ s/\s+//;
        $objid=$event."-".$series;
        $ste->finish;

    } elsif ( $site->{'site_datadriver'} =~ /mysql/ ) {

        $p= "INSERT into levents (series,dtable,dvalue,cdate) ";
        $p.=" values('$site->{'series'}','contacts',";
        $p.=" '$new->{'acct'}\t Create 39003-A', '$now') ";
        $stp=$dbh->prepare($p);
        $stp->execute;

        $e ="SELECT event,series from levents where dtable='contacts' and ";
        $e.=" dvalue='$new->{'acct'}\t Create 39003-A' and cdate='$now' ";
        $ste=$dbh->prepare($e);
        $ste->execute;
        #$nume=$ste->rows;
        ($event, $series)=$ste->fetchrow;
        $objid=$event."-".$series;  

    } elsif ( $site->{'site_datadriver'} =~ /Pg/ ) {

         my $p= "INSERT into levents (event,series,dtable,dvalue,cdate) ";
         $p.=" values(NEXTVAL('event'),'$site->{'series'}','contacts',";
         $p.=" '$new->{'acct'}\t Create 39003-A', '$now') ";
         $stp=$dbh->prepare($p);
         $stp->execute;
         $stp->finish;

         $q="SELECT CURRVAL('event') FROM levents where dtable='contacts' and dvalue='$new->{'acct'}\t Create 39003-A' and cdate='$now'";
         $stp=$dbh->prepare($q);
         $stp->execute;
         $event=$stp->fetchrow;
         $stp->finish;

         my $e ="SELECT series from levents where dtable='contacts' and ";
         $e.=" dvalue='$new->{'acct'}\t Create 39003-A' and cdate='$now' ";
         $ste=$dbh->prepare($e);
         $ste->execute;
         my $series=$ste->fetchrow;
         $series =~ s/\s+//;
         $objid=$event."-".$series;
         $ste->finish;    
    }

    return $objid; #
}

sub wel_create_user_acct {
	my ($dbh, $new, $objid, $FORM) = @_;	
	my $now = time();
	my $hint = $dbh->quote($new->{'hint_inp'});
	
	my $q="INSERT into contacts ";
	$q.=" (objid,objtype,uname,passwd,hint,email_ok,cdate,mdate) ";
	$q.=" VALUES ('$objid','39003-A','$new->{'acct'}', ";
	$q.=" '$new->{'passwd1_inp'}',$hint";
	$q.=",'$FORM->{'new.email_ok_inp'}',$now,$now)";
#	$dbh->do( $q );
        $sth = $dbh->prepare( $q );
        $sth->execute;
	return;
}

sub wel_update_sid {
	my ($FORM, $dbh) = @_;
	
	my $sid = $FORM->{'sid'};
	return unless $sid;
        $sid=&os_unhash(pack(H16,$sid), $site->{'hash_key'}) if ($sid && length($sid) > 11);	
	my $q="SELECT custid,sid,status,mdate from sessions where sid= $sid";
	my $sts = $dbh->prepare( $q );
	$sts->execute;
	my $h = $sts->fetchrow_hashref(NAME_lc);
        $sts->finish;
	return $h;
}

sub wel_update_session {
	my ($dbh, $sid, $custid) = @_;
	return unless $sid;

        $sid=&os_unhash(pack(H16,$sid), $site->{'hash_key'}) if ($sid && length($sid) > 11);
	
	my $now = time();
    my $q="UPDATE sessions SET mdate=$now";
	$q .= qq{, custid = '$custid'} if ($custid);
	$q .= qq{ where sid=$sid};

#	$dbh->do( $q );
        $sth = $dbh->prepare( $q );
        $sth->execute;
	return;
}


sub wel_check_uname {
	my ($dbh, $custid) = @_;
	
	my $q="SELECT uname from contacts where objid='$custid' ";
    my $sts = $dbh->prepare( $q );
    $sts->execute;
    return $sts->fetchrow;
}
        

sub wel_send_mail {
	my ($site, $new, %script_msg) = @_;
	
    my $mailprog = $site->{'mailserver'};
    my $title= $script_msg{9};

    open(MAIL,"|$mailprog -t");

    print MAIL "$script_msg{10}: $new->{'email_inp'}\n";
    print MAIL "$script_msg{11}: $site->{'customer_service_email_address'}\n";

    print MAIL "$script_msg{12}: $title\n\n";

    print MAIL "$script_msg{13} $site->{'sitename'}\n";
    print MAIL "$script_msg{14}:\n\n"; 
    print MAIL "$script_msg{15}: $new->{'email_inp'}\n";
##    print MAIL "$script_msg{16}: $new->{'passwd1_inp'}\n\n";
 
    print MAIL "$script_msg{24}\n";
    print MAIL "$script_msg{17} $site->{'customer_service_email_address'}.\n\n";
    print MAIL "$script_msg{18} http://$site->{'siteurl'}\n";

    close (MAIL);
}

sub wel_message {
	# check the language that is being used and open appropriate
	# message language file or get messages below
	my ($relpath, $om_render_lang, $scriptname) = @_;
	my %script_msg;
	
    if ($om_render_lang eq "eng" ) {

    	$script_msg{1}="Password too short!";
    	$script_msg{2}="A valid email address is required!";
    	$script_msg{3}="You need a valid email with your password";
    	$script_msg{4}="Matches on";
    	$script_msg{5}="Error - That address is already registered!";
    	$script_msg{6}="Please select a password!";
    	$script_msg{7}="Passwords are at least 5 characters!";
    	$script_msg{8}="Error - your passwords do not match";
    	$script_msg{9}="Your account has been created.";
    	$script_msg{10}="To";
    	$script_msg{11}="From";
    	$script_msg{12}="Subject";
    	$script_msg{13}="You have created an account with";
    	$script_msg{14}="using the following information";
    	$script_msg{15}="Email address";
    	$script_msg{16}="Password";
    	$script_msg{24}="If you have any problems or questions contact";
    	$script_msg{17}="customer service at";
    	$script_msg{18}="Thank you for visiting";
    	$script_msg{19}="Invalid login! Please try again.";
    	$script_msg{20}="Error - Invalid email address!";
    	$script_msg{21}="Error - No email address entered!";
    	$script_msg{22}="Error - Wrong email address format, need \@";
		$script_msg{23}="Your session id has been reset by the system. Please try to login again.";

    } else {
		open( EM, "<$relpath/bits/$om_render_lang/system/$scriptname.$om_render_lang.messages");
    		while(<EM>) {
    			chomp;
    			next if /^\#/;
    			next if (! $_);
    			my @temp=split /\t/;
    			$script_msg{$temp[0]}=$temp[1];
			}
		close(EM);
	}
	
	return %script_msg;
}

1;
